@@ -1,6 +1,6 @@
 //
 // This file is part of Gambit
-// Copyright (c) 1994-2010, The Gambit Project (http://www.gambit-project.org)
+// Copyright (c) 1994-2014, The Gambit Project (http://www.gambit-project.org)
 //
 // FILE: src/gui/dlnfglogit.cc
 // Dialog for monitoring progress of logit equilibrium computation
@@ -26,6 +26,7 @@
 #ifndef WX_PRECOMP
 #include <wx/wx.h>
 #endif  // WX_PRECOMP
+#include <wx/stdpaths.h>
 #include <wx/txtstrm.h>
 #include <wx/tokenzr.h>
 #include <wx/process.h>
@@ -72,14 +73,14 @@ public:
   
 void LogitMixedBranch::AddProfile(const wxString &p_text)
 {
-  MixedStrategyProfile<double> profile(m_doc->GetGame());
+  MixedStrategyProfile<double> profile(m_doc->GetGame()->NewMixedStrategyProfile(0.0));
 
   wxStringTokenizer tok(p_text, wxT(","));
 
-  m_lambdas.Append((double) ToNumber(std::string((const char *) tok.GetNextToken().mb_str())));
+  m_lambdas.Append((double) lexical_cast<Rational>(std::string((const char *) tok.GetNextToken().mb_str())));
 
-  for (int i = 1; i <= profile.Length(); i++) {
-    profile[i] = ToNumber(std::string((const char *) tok.GetNextToken().mb_str()));
+  for (int i = 1; i <= profile.MixedProfileLength(); i++) {
+    profile[i] = lexical_cast<Rational>(std::string((const char *) tok.GetNextToken().mb_str()));
   }
 
   m_profiles.Append(profile);
@@ -166,14 +167,14 @@ wxString LogitMixedSheet::GetCellValue(const wxSheetCoords &p_coords)
   }
 
   if (p_coords.GetCol() == 0) {
-    return wxString(ToText(m_branch.GetLambda(p_coords.GetRow()+1),
+    return wxString(lexical_cast<std::string>(m_branch.GetLambda(p_coords.GetRow()+1),
 			   m_doc->GetStyle().NumDecimals()).c_str(),
 		    *wxConvCurrent);
   }
   else {
     const MixedStrategyProfile<double> &profile = 
       m_branch.GetProfile(p_coords.GetRow()+1);
-    return wxString(ToText(profile[p_coords.GetCol()],
+    return wxString(lexical_cast<std::string>(profile[p_coords.GetCol()],
 				   m_doc->GetStyle().NumDecimals()).c_str(), 
 		    *wxConvCurrent);
   }
@@ -266,7 +267,7 @@ LogitBranchDialog::LogitBranchDialog(wxWindow *p_parent,
 
 class gbtLogitPlotCtrl : public wxPlotCtrl {
 private:
-  gbtGameDocument *m_doc;
+  // gbtGameDocument *m_doc;
   double m_scaleFactor;
 
   /// Overriding x (lambda) axis labeling
@@ -285,7 +286,7 @@ public:
 
 gbtLogitPlotCtrl::gbtLogitPlotCtrl(wxWindow *p_parent, 
 				   gbtGameDocument *p_doc)
-  : wxPlotCtrl(p_parent), m_doc(p_doc), m_scaleFactor(1.0)
+  : wxPlotCtrl(p_parent), /* m_doc(p_doc), */ m_scaleFactor(1.0)
 {
   SetAxisLabelColour(*wxBLUE);
   wxFont labelFont(8, wxSWISS, wxNORMAL, wxBOLD);
@@ -382,7 +383,7 @@ private:
 public:
   gbtLogitPlotStrategyList(wxWindow *p_parent, gbtGameDocument *p_doc);
 
-  bool IsShown(int p_index)
+  bool IsStrategyShown(int p_index)
   { return GetCellValue(wxSheetCoords(p_index-1, 2)) == wxT("1"); }
 };
 
@@ -509,7 +510,7 @@ void LogitPlotPanel::Plot(void)
   m_plotCtrl->DeleteCurve(-1);
 
   for (int st = 1; st <= m_doc->GetGame()->MixedProfileLength(); st++) {
-    if (!m_plotStrategies->IsShown(st)) continue;
+    if (!m_plotStrategies->IsStrategyShown(st)) continue;
 
     wxPlotData *curve = new wxPlotData(m_branch.NumPoints());
 
@@ -655,24 +656,43 @@ LogitMixedDialog::LogitMixedDialog(wxWindow *p_parent,
   m_toolBar->SetMargins(4, 4);
   m_toolBar->SetToolBitmapSize(wxSize(24, 24));
 
-  m_toolBar->AddTool(wxID_SAVE, wxBitmap(savedata_xpm), wxNullBitmap, false,
-		     -1, -1, 0,
-		     _("Save the correspondence to a CSV file"),
-		     _("Save the correspondence to a CSV file"));
+  m_toolBar->AddTool(wxID_SAVE,
+                     wxEmptyString,
+                     wxBitmap(savedata_xpm),
+                     wxNullBitmap,
+                     wxITEM_NORMAL,
+		                 _("Save the correspondence to a CSV file"),
+		                 _("Save the correspondence to a CSV file"));
+
   m_toolBar->EnableTool(wxID_SAVE, false);
-  m_toolBar->AddTool(GBT_MENU_VIEW_DATA, wxBitmap(datasrc_xpm),
-		     wxNullBitmap, false, -1, -1, 0,
-		     _("View the points in the correspondence"),
-		     _("View the points in the correspondence"));
+
+  m_toolBar->AddTool(GBT_MENU_VIEW_DATA,
+                     wxEmptyString,
+                     wxBitmap(datasrc_xpm),
+                     wxNullBitmap,
+                     wxITEM_NORMAL,
+		                 _("View the points in the correspondence"),
+		                 _("View the points in the correspondence"));
+
   m_toolBar->EnableTool(GBT_MENU_VIEW_DATA, false);
 
-  m_toolBar->AddTool(wxID_PRINT, wxBitmap(print_xpm), wxNullBitmap, false,
-		     -1, -1, 0, _("Print the graph"), _("Print the graph"));
+  m_toolBar->AddTool(wxID_PRINT,
+                     wxEmptyString,
+                     wxBitmap(print_xpm),
+                     wxNullBitmap,
+                     wxITEM_NORMAL,
+                     _("Print the graph"),
+                     _("Print the graph"));
 
   m_toolBar->AddSeparator();
-  m_toolBar->AddTool(GBT_MENU_VIEW_ZOOMFIT, wxBitmap(zoomfit_xpm),
-		     wxNullBitmap, false, -1, -1, 0,
-		     _("Show the whole graph"), _("Show the whole graph"));
+
+  m_toolBar->AddTool(GBT_MENU_VIEW_ZOOMFIT,
+                     wxEmptyString,
+                     wxBitmap(zoomfit_xpm),
+                     wxNullBitmap,
+                     wxITEM_NORMAL,
+                     _("Show the whole graph"),
+                     _("Show the whole graph"));
 
   m_toolBar->AddControl(new wxStaticText(m_toolBar, wxID_STATIC,
 					 wxT("Graph scaling:")));
@@ -717,14 +737,15 @@ void LogitMixedDialog::Start(void)
   m_process->Redirect();
 
 #ifdef __WXMAC__
-  m_pid = wxExecute(wxT("/usr/local/bin/gambit-logit -S"),
+  m_pid = wxExecute(wxStandardPaths::Get().GetExecutablePath() + 
+		    wxT("-logit -S"),
                     wxEXEC_ASYNC, m_process);
 #else
   m_pid = wxExecute(wxT("gambit-logit -S"), wxEXEC_ASYNC, m_process);
 #endif // __WXMAC__
   
   std::ostringstream s;
-  m_doc->GetGame()->WriteNfgFile(s);
+  m_doc->GetGame()->Write(s, "nfg");
   wxString str(wxString(s.str().c_str(), *wxConvCurrent));
   
   // It is possible that the whole string won't write on one go, so
@@ -827,9 +848,9 @@ void LogitMixedDialog::OnStop(wxCommandEvent &)
 void LogitMixedDialog::OnSave(wxCommandEvent &)
 {
   wxFileDialog dialog(this, _("Choose file"), wxT(""), wxT(""),
-		      wxT("CSV files (*.csv)|*.csv|"
-			  "All files (*.*)|*.*"),
-		      wxSAVE | wxOVERWRITE_PROMPT);
+		      wxT("CSV files (*.csv)|*.csv|")
+			  wxT("All files (*.*)|*.*"),
+		      wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
   if (dialog.ShowModal() == wxID_OK) {
     std::ofstream file((const char *) dialog.GetPath().mb_str());
