@@ -1,6 +1,6 @@
 //
 // This file is part of Gambit
-// Copyright (c) 1994-2010, The Gambit Project (http://www.gambit-project.org)
+// Copyright (c) 1994-2014, The Gambit Project (http://www.gambit-project.org)
 //
 // FILE: src/gui/efglayout.cc
 // Implementation of tree layout representation
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
 
-#include <math.h>
+#include <cmath>
+#include <algorithm>    // for std::min, std::max
+
 #include <wx/wxprec.h>
 #ifndef WX_PRECOMP
 #include <wx/wx.h>
@@ -253,12 +255,12 @@ static wxPoint DrawFraction(wxDC &p_dc, wxPoint p_point,
   p_dc.SetFont(wxFont(7, wxSWISS, wxNORMAL, wxBOLD));
   
   int numWidth, numHeight;
-  wxString num = wxString(ToText(p_value.numerator()).c_str(),
+  wxString num = wxString(lexical_cast<std::string>(p_value.numerator()).c_str(),
 			  *wxConvCurrent);
   p_dc.GetTextExtent(num, &numWidth, &numHeight);
 
   int denWidth, denHeight;
-  wxString den = wxString(ToText(p_value.denominator()).c_str(),
+  wxString den = wxString(lexical_cast<std::string>(p_value.denominator()).c_str(),
 			  *wxConvCurrent);
   p_dc.GetTextExtent(den, &denWidth, &denHeight);
 
@@ -303,7 +305,7 @@ void gbtNodeEntry::DrawOutcome(wxDC &p_dc, bool p_noHints) const
 
     std::string payoff = outcome->GetPayoff<std::string>(pl);
 
-    if (payoff.find('/') != (unsigned int) -1) {
+    if (payoff.find('/') != std::string::npos) {
       p_dc.SetPen(wxPen(m_style->GetPlayerColor(pl), 1, wxSOLID));
       int oldX = point.x;
       point = DrawFraction(p_dc, point, outcome->GetPayoff<Gambit::Rational>(pl));
@@ -351,7 +353,7 @@ bool gbtNodeEntry::NodeHitTest(int p_x, int p_y) const
 
 gbtTreeLayout::gbtTreeLayout(gbtEfgDisplay *p_parent, gbtGameDocument *p_doc)
   : gbtGameView(p_doc), 
-    m_parent(p_parent), m_infosetSpacing(40),
+    /* m_parent(p_parent),*/ m_infosetSpacing(40),
     c_leftMargin(20), c_topMargin(40)
 { }
 
@@ -492,7 +494,7 @@ wxString gbtTreeLayout::CreateBranchLabel(const gbtNodeEntry *p_entry,
 		      *wxConvCurrent);
     case GBT_BRANCH_LABEL_PROBS:
       if (parent->GetPlayer() && parent->GetPlayer()->IsChance()) {
-	return wxString(parent->GetInfoset()->GetActionProb<std::string>(p_entry->GetChildNumber()).c_str(),
+	return wxString(parent->GetInfoset()->GetActionProb(p_entry->GetChildNumber(), "").c_str(),
 			*wxConvCurrent);
       }
       else if (m_doc->NumProfileLists() == 0) {
@@ -582,7 +584,7 @@ Gambit::GameNode gbtTreeLayout::NextSameLevel(Gambit::GameNode p_node) const
   return 0;
 }
 
-int gbtTreeLayout::LayoutSubtree(Gambit::GameNode p_node, const Gambit::BehavSupport &p_support,
+int gbtTreeLayout::LayoutSubtree(Gambit::GameNode p_node, const Gambit::BehaviorSupportProfile &p_support,
 				 int &p_maxy, int &p_miny, int &p_ycoord)
 {
   int y1 = -1, yn = 0;
@@ -654,8 +656,8 @@ int gbtTreeLayout::LayoutSubtree(Gambit::GameNode p_node, const Gambit::BehavSup
   }
   entry->SetBranchLength(settings.BranchLength());
 
-  p_maxy = Gambit::max(entry->Y(), p_maxy);
-  p_miny = Gambit::min(entry->Y(), p_miny);
+  p_maxy = std::max(entry->Y(), p_maxy);
+  p_miny = std::min(entry->Y(), p_miny);
     
   return entry->Y();
 }
@@ -722,7 +724,7 @@ void gbtTreeLayout::CheckInfosetEntry(gbtNodeEntry *e)
     e1 = m_nodeList[pos];
     // Find the max num for this level
     if (e->GetLevel() == e1->GetLevel())  {
-      num = Gambit::max(e1->GetSublevel(), num);
+      num = std::max(e1->GetSublevel(), num);
     }
   }
   num++;
@@ -730,7 +732,7 @@ void gbtTreeLayout::CheckInfosetEntry(gbtNodeEntry *e)
   e->SetNextMember(infoset_entry);
 }
 
-void gbtTreeLayout::FillInfosetTable(Gambit::GameNode n, const Gambit::BehavSupport &cur_sup)
+void gbtTreeLayout::FillInfosetTable(Gambit::GameNode n, const Gambit::BehaviorSupportProfile &cur_sup)
 {
   const gbtStyle &draw_settings = m_doc->GetStyle();
   gbtNodeEntry *entry = GetNodeEntry(n);
@@ -762,7 +764,7 @@ void gbtTreeLayout::UpdateTableInfosets(void)
   // find the max e->num for each level
   for (int pos = 1; pos <= m_nodeList.Length(); pos++) {
     gbtNodeEntry *entry = m_nodeList[pos];
-    nums[entry->GetLevel()] = Gambit::max(entry->GetSublevel() + 1,
+    nums[entry->GetLevel()] = std::max(entry->GetSublevel() + 1,
 					  nums[entry->GetLevel()]);
   }
     
@@ -779,7 +781,7 @@ void gbtTreeLayout::UpdateTableInfosets(void)
 		  (nums[entry->GetLevel()-1] +
 		   entry->GetSublevel()) * m_infosetSpacing);
     }
-    m_maxX = Gambit::max(m_maxX, entry->X() + entry->GetSize());
+    m_maxX = std::max(m_maxX, entry->X() + entry->GetSize());
   }
 }
 
@@ -792,7 +794,7 @@ void gbtTreeLayout::UpdateTableParents(void)
   }
 }
 
-void gbtTreeLayout::Layout(const Gambit::BehavSupport &p_support)
+void gbtTreeLayout::Layout(const Gambit::BehaviorSupportProfile &p_support)
 {
   // Kinda kludgey; probably should query draw settings whenever needed.
   m_infosetSpacing = 
@@ -819,7 +821,7 @@ void gbtTreeLayout::Layout(const Gambit::BehavSupport &p_support)
   m_maxY = maxy + 25;
 }
 
-void gbtTreeLayout::BuildNodeList(Gambit::GameNode p_node, const Gambit::BehavSupport &p_support,
+void gbtTreeLayout::BuildNodeList(Gambit::GameNode p_node, const Gambit::BehaviorSupportProfile &p_support,
 				  int p_level)
 {
   gbtNodeEntry *entry = new gbtNodeEntry(p_node);
@@ -847,10 +849,10 @@ void gbtTreeLayout::BuildNodeList(Gambit::GameNode p_node, const Gambit::BehavSu
       BuildNodeList(p_node->GetChild(i), p_support, p_level + 1);
     }
   }
-  m_maxLevel = Gambit::max(p_level, m_maxLevel);
+  m_maxLevel = std::max(p_level, m_maxLevel);
 }
 
-void gbtTreeLayout::BuildNodeList(const Gambit::BehavSupport &p_support)
+void gbtTreeLayout::BuildNodeList(const Gambit::BehaviorSupportProfile &p_support)
 {
   while (m_nodeList.Length() > 0) {
     delete m_nodeList.Remove(1);
@@ -882,13 +884,13 @@ void gbtTreeLayout::GenerateLabels(void)
 
       Gambit::GameNode parent = entry->GetNode()->GetParent();
       if (parent->GetPlayer()->IsChance()) {
-	entry->SetActionProb(parent->GetInfoset()->GetActionProb<double>(entry->GetChildNumber()));
+	entry->SetActionProb(parent->GetInfoset()->GetActionProb(entry->GetChildNumber(), (double) 0));
       }
       else {
 	int profile = m_doc->GetCurrentProfile();
 	if (profile > 0) {
 	  try {
-	    entry->SetActionProb((double) Gambit::ToNumber(m_doc->GetProfiles().GetActionProb(parent, entry->GetChildNumber())));
+	    entry->SetActionProb((double) Gambit::lexical_cast<Gambit::Rational>(m_doc->GetProfiles().GetActionProb(parent, entry->GetChildNumber())));
 	  }
 	  catch (ValueException &) {
 	    // This occurs when the probability is undefined
