@@ -1,6 +1,6 @@
 //
 // This file is part of Gambit
-// Copyright (c) 1994-2010, The Gambit Project (http://www.gambit-project.org)
+// Copyright (c) 1994-2014, The Gambit Project (http://www.gambit-project.org)
 //
 // FILE: src/gui/gamedoc.cc
 // Implementation of game document class
@@ -30,7 +30,7 @@
 #include <wx/filename.h>       // used to create temp files for undo/redo
 
 #include "libgambit/libgambit.h"
-#include "tinyxml.h"    // for XML parser for LoadDocument()
+#include "libgambit/tinyxml.h"    // for XML parser for LoadDocument()
 
 #include "gambit.h"     // for wxGetApp()
 #include "gamedoc.h"
@@ -60,9 +60,9 @@ void gbtBehavDominanceStack::SetStrict(bool p_strict)
 void gbtBehavDominanceStack::Reset(void)
 {
   for (int i = 1; i <= m_supports.Length(); delete m_supports[i++]);
-  m_supports = Gambit::Array<Gambit::BehavSupport *>();
+  m_supports = Gambit::Array<Gambit::BehaviorSupportProfile *>();
   if (m_doc->IsTree()) {
-    m_supports.Append(new Gambit::BehavSupport(m_doc->GetGame()));
+    m_supports.Append(new Gambit::BehaviorSupportProfile(m_doc->GetGame()));
     m_current = 1;
   }
   m_noFurther = false;
@@ -85,11 +85,11 @@ bool gbtBehavDominanceStack::NextLevel(void)
   }
   
   std::ostringstream gnull;
-  Gambit::BehavSupport newSupport = 
+  Gambit::BehaviorSupportProfile newSupport = 
     m_supports[m_current]->Undominated(m_strict, true, players, gnull);
 
   if (newSupport != *m_supports[m_current]) {
-    m_supports.Append(new Gambit::BehavSupport(newSupport));
+    m_supports.Append(new Gambit::BehaviorSupportProfile(newSupport));
     m_current++;
     return true;
   }
@@ -135,13 +135,10 @@ void gbtStrategyDominanceStack::SetStrict(bool p_strict)
 void gbtStrategyDominanceStack::Reset(void)
 {
   for (int i = 1; i <= m_supports.Length(); delete m_supports[i++]);
-  m_supports = Gambit::Array<Gambit::StrategySupport *>();
-
-  if (m_doc->GetGame()->HasComputedValues()) {
-    m_supports.Append(new Gambit::StrategySupport(m_doc->GetGame()));
-    m_current = 1;
-    m_noFurther = false;
-  }
+  m_supports = Gambit::Array<Gambit::StrategySupportProfile *>();
+  m_supports.Append(new Gambit::StrategySupportProfile(m_doc->GetGame()));
+  m_current = 1;
+  m_noFurther = false;
 }
 
 bool gbtStrategyDominanceStack::NextLevel(void)
@@ -160,11 +157,11 @@ bool gbtStrategyDominanceStack::NextLevel(void)
     players.Append(pl);
   }
 
-  Gambit::StrategySupport newSupport = 
+  Gambit::StrategySupportProfile newSupport = 
     m_supports[m_current]->Undominated(m_strict, players);
 
   if (newSupport != *m_supports[m_current]) {
-    m_supports.Append(new Gambit::StrategySupport(newSupport));
+    m_supports.Append(new Gambit::StrategySupportProfile(newSupport));
     m_current++;
     return true;
   }
@@ -195,7 +192,6 @@ gbtGameDocument::gbtGameDocument(Gambit::Game p_game)
     m_behavSupports(this, true), m_stratSupports(this, true),
     m_currentProfileList(0)
 {
-  m_game->Canonicalize();
   wxGetApp().AddDocument(this);
 
   std::ostringstream s;
@@ -272,8 +268,8 @@ bool gbtGameDocument::LoadDocument(const wxString &p_filename,
       bool isFloat = false;
       for (TiXmlNode *profile = analysis->FirstChild("profile");
 	   profile; profile = profile->NextSiblingElement()) {
-	if (std::string(profile->FirstChild()->Value()).find('.') != (unsigned int) -1 ||
-	    std::string(profile->FirstChild()->Value()).find('e') != (unsigned int) -1) {
+	if (std::string(profile->FirstChild()->Value()).find('.') != std::string::npos ||
+	    std::string(profile->FirstChild()->Value()).find('e') != std::string::npos) {
 	  isFloat = true;
 	  break;
 	}
@@ -340,13 +336,13 @@ void gbtGameDocument::SaveDocument(std::ostream &p_file) const
 
   if (m_game->IsTree()) {
     p_file << "<efgfile>\n";
-    m_game->WriteEfgFile(p_file);
+    m_game->Write(p_file, "efg");
     p_file << "</efgfile>\n";
 
   }
   else {
     p_file << "<nfgfile>\n";
-    m_game->WriteNfgFile(p_file);
+    m_game->Write(p_file, "nfg");
     p_file << "</nfgfile>\n";
   }
 
@@ -363,7 +359,6 @@ void gbtGameDocument::UpdateViews(gbtGameModificationType p_modifications)
 {
   if (p_modifications != GBT_DOC_MODIFIED_NONE) {
     m_modified = true;
-    m_game->Canonicalize();
     m_redoList = Gambit::List<std::string>();
 
     std::ostringstream s;
@@ -397,12 +392,17 @@ void gbtGameDocument::PostPendingChanges(void)
 void gbtGameDocument::BuildNfg(void)
 { 
   if (m_game->IsTree()) {
-    m_game->BuildComputedValues();
     m_stratSupports.Reset();
     for (int i = 1; i <= m_profiles.Length(); m_profiles[i++]->BuildNfg());
   }
 }
 
+void gbtGameDocument::SetStyle(const gbtStyle &p_style)
+{
+  m_style = p_style;
+  UpdateViews(GBT_DOC_MODIFIED_VIEWS);
+}
+
 //
 // A word about the undo and redo features:
 // We store a list of the textual representation of games.  We don't
@@ -597,3 +597,248 @@ void gbtGameDocument::SetSelectNode(Gambit::GameNode p_node)
   UpdateViews(GBT_DOC_MODIFIED_VIEWS);
 }
 
+//======================================================================
+// Commands for model part of MVC architecture start here.
+//======================================================================
+
+void gbtGameDocument::DoSave(const wxString &p_filename)
+{
+  std::ofstream file(static_cast<const char *>(p_filename.mb_str()));
+  SaveDocument(file);
+  m_filename = p_filename;
+  SetModified(false);
+  UpdateViews(GBT_DOC_MODIFIED_NONE);
+}
+
+void gbtGameDocument::DoExportEfg(const wxString &p_filename)
+{
+  std::ofstream file(static_cast<const char *>(p_filename.mb_str()));
+  m_game->Write(file, "efg");
+  UpdateViews(GBT_DOC_MODIFIED_NONE);
+}
+
+void gbtGameDocument::DoExportNfg(const wxString &p_filename)
+{
+  std::ofstream file(static_cast<const char *>(p_filename.mb_str()));
+  BuildNfg();
+  m_game->Write(file, "nfg");
+  UpdateViews(GBT_DOC_MODIFIED_NONE);
+}
+
+
+void gbtGameDocument::DoSetTitle(const wxString &p_title, 
+				 const wxString &p_comment)
+{
+  m_game->SetTitle(static_cast<const char *>(p_title.mb_str()));
+  m_game->SetComment(static_cast<const char *>(p_comment.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_LABELS);
+}
+
+void gbtGameDocument::DoNewPlayer(void)
+{
+  GamePlayer player = m_game->NewPlayer();
+  player->SetLabel("Player " + 
+		   lexical_cast<std::string>(player->GetNumber()));
+  if (!m_game->IsTree()) {
+    player->GetStrategy(1)->SetLabel("1");
+  }
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoSetPlayerLabel(GamePlayer p_player,
+				       const wxString &p_label)
+{
+  p_player->SetLabel(static_cast<const char *>(p_label.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_LABELS);
+}
+
+void gbtGameDocument::DoNewStrategy(GamePlayer p_player)
+{
+  GameStrategy strategy = p_player->NewStrategy();
+  strategy->SetLabel(lexical_cast<std::string>(strategy->GetNumber()));
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoDeleteStrategy(GameStrategy p_strategy)
+{
+  p_strategy->DeleteStrategy();
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoSetStrategyLabel(GameStrategy p_strategy,
+					 const wxString &p_label)
+{
+  p_strategy->SetLabel(static_cast<const char *>(p_label.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_LABELS);
+}
+
+void gbtGameDocument::DoSetInfosetLabel(GameInfoset p_infoset,
+					const wxString &p_label)
+{
+  p_infoset->SetLabel(static_cast<const char *>(p_label.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_LABELS);
+}
+
+void gbtGameDocument::DoSetActionLabel(GameAction p_action,
+				       const wxString &p_label)
+{
+  p_action->SetLabel(static_cast<const char *>(p_label.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_LABELS);
+}
+
+void gbtGameDocument::DoSetActionProb(GameInfoset p_infoset,
+				      unsigned int p_action,
+				      const wxString &p_prob)
+{
+  p_infoset->SetActionProb(p_action, static_cast<const char *>(p_prob.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_PAYOFFS);
+}
+
+void gbtGameDocument::DoSetInfoset(GameNode p_node, GameInfoset p_infoset)
+{
+  p_node->SetInfoset(p_infoset);
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoLeaveInfoset(GameNode p_node)
+{
+  p_node->LeaveInfoset();
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoRevealAction(GameInfoset p_infoset, GamePlayer p_player)
+{
+  p_infoset->Reveal(p_player);
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoInsertAction(GameNode p_node)
+{
+  if (!p_node || !p_node->GetInfoset())  return;
+  GameAction action = p_node->GetInfoset()->InsertAction();
+  action->SetLabel(lexical_cast<std::string>(action->GetNumber()));
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoSetNodeLabel(GameNode p_node, const wxString &p_label)
+{
+  p_node->SetLabel(static_cast<const char *>(p_label.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_LABELS);
+}
+
+
+void gbtGameDocument::DoAppendMove(GameNode p_node, GameInfoset p_infoset)
+{
+  p_node->AppendMove(p_infoset);
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoInsertMove(GameNode p_node, GamePlayer p_player,
+				   unsigned int p_actions)
+{
+  GameInfoset infoset = p_node->InsertMove(p_player, p_actions);
+  for (int act = 1; act <= infoset->NumActions(); act++) {
+    infoset->GetAction(act)->SetLabel(lexical_cast<std::string>(act));
+  }
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoInsertMove(GameNode p_node, GameInfoset p_infoset)
+{
+  p_node->InsertMove(p_infoset);
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoCopyTree(GameNode p_destNode, GameNode p_srcNode)
+{
+  p_destNode->CopyTree(p_srcNode);
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoMoveTree(GameNode p_destNode, GameNode p_srcNode)
+{
+  p_destNode->MoveTree(p_srcNode);
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoDeleteParent(GameNode p_node)
+{
+  if (!p_node || !p_node->GetParent()) return;
+  p_node->DeleteParent();
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoDeleteTree(GameNode p_node)
+{
+  p_node->DeleteTree();
+  UpdateViews(GBT_DOC_MODIFIED_GAME);
+}
+
+void gbtGameDocument::DoSetPlayer(GameInfoset p_infoset, GamePlayer p_player)
+{
+  if (!p_player->IsChance() && !p_infoset->GetPlayer()->IsChance()) {
+    // Currently don't support switching nodes to/from chance player
+    p_infoset->SetPlayer(p_player);
+    UpdateViews(GBT_DOC_MODIFIED_GAME);
+  }
+}
+
+void gbtGameDocument::DoSetPlayer(GameNode p_node, GamePlayer p_player)
+{
+  if (!p_player->IsChance() && !p_node->GetPlayer()->IsChance()) {
+    // Currently don't support switching nodes to/from chance player
+    p_node->GetInfoset()->SetPlayer(p_player);
+    UpdateViews(GBT_DOC_MODIFIED_GAME);
+  }
+}
+
+void gbtGameDocument::DoNewOutcome(GameNode p_node)
+{
+  p_node->SetOutcome(m_game->NewOutcome());
+  UpdateViews(GBT_DOC_MODIFIED_PAYOFFS);
+}
+
+void gbtGameDocument::DoNewOutcome(const PureStrategyProfile &p_profile)
+{
+  p_profile->SetOutcome(m_game->NewOutcome());
+  UpdateViews(GBT_DOC_MODIFIED_PAYOFFS);
+}
+
+void gbtGameDocument::DoSetOutcome(GameNode p_node, GameOutcome p_outcome)
+{
+  p_node->SetOutcome(p_outcome);
+  UpdateViews(GBT_DOC_MODIFIED_PAYOFFS);
+}
+
+void gbtGameDocument::DoRemoveOutcome(GameNode p_node)
+{
+  if (!p_node || !p_node->GetOutcome()) return;
+  p_node->SetOutcome(0);
+  UpdateViews(GBT_DOC_MODIFIED_PAYOFFS);
+}
+
+void gbtGameDocument::DoCopyOutcome(GameNode p_node, GameOutcome p_outcome)
+{
+  GameOutcome outcome = m_game->NewOutcome();
+  outcome->SetLabel("Outcome" + lexical_cast<std::string>(outcome->GetNumber()));
+  for (int pl = 1; pl <= m_game->NumPlayers(); pl++) {
+    outcome->SetPayoff(pl, p_outcome->GetPayoff<std::string>(pl));
+  }
+  p_node->SetOutcome(outcome);
+  UpdateViews(GBT_DOC_MODIFIED_PAYOFFS);
+}
+
+void gbtGameDocument::DoSetPayoff(GameOutcome p_outcome, int p_player,
+				  const wxString &p_value)
+{
+  p_outcome->SetPayoff(p_player, 
+		       static_cast<const char *>(p_value.mb_str()));
+  UpdateViews(GBT_DOC_MODIFIED_PAYOFFS);
+}
+
+void gbtGameDocument::DoAddOutput(gbtAnalysisOutput &p_list,
+				  const wxString &p_output)
+{
+  p_list.AddOutput(p_output);
+  UpdateViews(GBT_DOC_MODIFIED_NONE);
+}
