@@ -1,6 +1,6 @@
 //
 // This file is part of Gambit
-// Copyright (c) 1994-2010, The Gambit Project (http://www.gambit-project.org)
+// Copyright (c) 1994-2014, The Gambit Project (http://www.gambit-project.org)
 //
 // FILE: src/gui/gameframe.cc
 // Implementation of frame containing views of a game
@@ -46,6 +46,7 @@
 #include "nfgpanel.h"
 #include "nfgprofile.h"
 
+#include "dlexcept.h"
 #include "dlgameprop.h"
 #include "dlnash.h"
 #include "dlnashmon.h"
@@ -276,25 +277,20 @@ gbtGameFrame::gbtGameFrame(wxWindow *p_parent, gbtGameDocument *p_doc)
   m_splitter = new wxSplitterWindow(this, -1);
   if (p_doc->IsTree()) {
     m_efgPanel = new gbtEfgPanel(m_splitter, p_doc);
+    m_efgPanel->Show(true);
+    m_splitter->Initialize(m_efgPanel);
+    m_nfgPanel = 0;
   }
   else {
     m_efgPanel = 0;
-  }
-
-  m_nfgPanel = new gbtNfgPanel(m_splitter, p_doc);
-  if (p_doc->IsTree()) {
-    m_nfgPanel->Show(false);
+    m_nfgPanel = new gbtNfgPanel(m_splitter, p_doc);
+    m_nfgPanel->Show(true);
+    m_splitter->Initialize(m_nfgPanel);
   }
 
   m_analysisPanel = new gbtAnalysisNotebook(m_splitter, p_doc);
   m_analysisPanel->Show(false);
 
-  if (p_doc->IsTree()) {
-    m_splitter->Initialize(m_efgPanel);
-  }
-  else {
-    m_splitter->Initialize(m_nfgPanel);
-  }
   m_splitter->SetSashGravity(0.5);
   m_splitter->SetMinimumPaneSize(200);
 
@@ -312,10 +308,9 @@ gbtGameFrame::gbtGameFrame(wxWindow *p_parent, gbtGameDocument *p_doc)
   else {
     m_nfgPanel->SetFocus();
   }
-
+  
+  OnUpdate();
   Show(true);
-
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_NONE);
 }
 
 gbtGameFrame::~gbtGameFrame()
@@ -352,7 +347,7 @@ void gbtGameFrame::OnUpdate(void)
   GetToolBar()->EnableTool(wxID_UNDO, m_doc->CanUndo());
   menuBar->Enable(wxID_REDO, m_doc->CanRedo());
   GetToolBar()->EnableTool(wxID_REDO, m_doc->CanRedo());
-  menuBar->Enable(GBT_MENU_EDIT_INSERT_MOVE, selectNode);
+  menuBar->Enable(GBT_MENU_EDIT_INSERT_MOVE, selectNode != 0);
   menuBar->Enable(GBT_MENU_EDIT_INSERT_ACTION,
 		  selectNode && selectNode->GetInfoset());
   menuBar->Enable(GBT_MENU_EDIT_REVEAL,
@@ -363,7 +358,7 @@ void gbtGameFrame::OnUpdate(void)
 		  selectNode && selectNode->GetParent());
   menuBar->Enable(GBT_MENU_EDIT_REMOVE_OUTCOME,
 		  selectNode && selectNode->GetOutcome());
-  menuBar->Enable(GBT_MENU_EDIT_NODE, selectNode);
+  menuBar->Enable(GBT_MENU_EDIT_NODE, selectNode != 0);
   menuBar->Enable(GBT_MENU_EDIT_MOVE, 
 		     selectNode && selectNode->GetInfoset());
 
@@ -554,8 +549,8 @@ void gbtGameFrame::MakeMenus(void)
   viewMenu->AppendSeparator();
     
   viewMenu->Append(GBT_MENU_VIEW_STRATEGIC, _("&Strategic game"),
-		   _("Display the reduced strategic representation "
-		     "of the game"), true);
+		   wxT("Display the reduced strategic representation ")
+		     wxT("of the game"), true);
   
   wxMenu *formatMenu = new wxMenu;
   AppendBitmapItem(formatMenu, GBT_MENU_FORMAT_LAYOUT, _("&Layout"),
@@ -600,85 +595,180 @@ void gbtGameFrame::MakeToolbar(void)
   toolBar->SetMargins(4, 4);
   toolBar->SetToolBitmapSize(wxSize(24, 24));
 
-  toolBar->AddTool(GBT_MENU_FILE_NEW_EFG, wxBitmap(newtree_xpm),
-		   wxNullBitmap, false, -1, -1, 0,
-		   _("Create a new extensive (tree) game"),
-		   _("Create a new extensive (tree) game"));
-  toolBar->AddTool(GBT_MENU_FILE_NEW_NFG, wxBitmap(newtable_xpm),
-		   wxNullBitmap, false, -1, -1, 0,
-		   _("Create a new strategic (table) game"),
-		   _("Create a new strategic (table) game"));
-  toolBar->AddTool(wxID_OPEN, wxBitmap(open_xpm), wxNullBitmap, false,
-		   -1, -1, 0, _("Open a file"), _("Open a saved game"));
-  toolBar->AddTool(wxID_SAVE, wxBitmap(save_xpm), wxNullBitmap, false,
-		   -1, -1, 0, _("Save this game"), _("Save this game"));
-  toolBar->AddTool(wxID_SAVEAS, wxBitmap(saveas_xpm), wxNullBitmap, false,
-		   -1, -1, 0, _("Save to a different file"), 
-		   _("Save this game to another file"));
-  toolBar->AddSeparator();
-
-  toolBar->AddTool(wxID_PRINT, wxBitmap(print_xpm), wxNullBitmap, false,
-		   -1, -1, 0, _("Print this game"), _("Print this game"));
-  toolBar->AddTool(wxID_PREVIEW, wxBitmap(preview_xpm), wxNullBitmap,
-		   false, -1, -1, 0, _("Print preview"),
-		   _("View a preview of the game printout"));
+  toolBar->AddTool(GBT_MENU_FILE_NEW_EFG, 
+                   wxEmptyString,
+                   wxBitmap(newtree_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+		               _("Create a new extensive (tree) game"),
+		               _("Create a new extensive (tree) game"),
+                   NULL);
+  toolBar->AddTool(GBT_MENU_FILE_NEW_NFG, 
+                   wxEmptyString,
+                   wxBitmap(newtable_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+		               _("Create a new strategic (table) game"),
+		               _("Create a new strategic (table) game"),
+                   NULL);
+  toolBar->AddTool(wxID_OPEN, 
+                   wxEmptyString,
+                   wxBitmap(open_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Open a file"),
+                   _("Open a file"),
+                   NULL);
+  toolBar->AddTool(wxID_SAVE,
+                   wxEmptyString,
+                   wxBitmap(save_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Save this game"), 
+                   _("Save this game"), 
+                   NULL);
+  toolBar->AddTool(wxID_SAVEAS, 
+                   wxEmptyString,
+                   wxBitmap(saveas_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Save to a different file"), 
+                   _("Save this game to another file"),
+                   NULL);
 
   toolBar->AddSeparator();
-  toolBar->AddTool(wxID_UNDO, wxBitmap(undo_xpm), wxNullBitmap,
-		   false, -1, -1, 0, _("Undo the last action"),
-		   _("Undo the last change to the game"));
-  toolBar->AddTool(wxID_REDO, wxBitmap(redo_xpm), wxNullBitmap,
-		   false, -1, -1, 0, _("Redo the undone action"),
-		   _("Redo the last undone change"));
+
+  toolBar->AddTool(wxID_PRINT, 
+                   wxEmptyString,
+                   wxBitmap(print_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Print this game"),
+                   _("Print this game"),
+                   NULL);
+  toolBar->AddTool(wxID_PREVIEW, 
+                   wxEmptyString,
+                   wxBitmap(preview_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Print preview"),
+                   _("View a preview of the game printout"),
+                   NULL);
 
   toolBar->AddSeparator();
-  toolBar->AddTool(GBT_MENU_EDIT_NEWPLAYER, wxBitmap(newplayer_xpm),
-		   wxNullBitmap, false, -1, -1, 0,
-		   _("Add a new player"), _("Add a new player to the game"));
+
+  toolBar->AddTool(wxID_UNDO, 
+                   wxEmptyString,
+                   wxBitmap(undo_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Undo the last action"),
+                   _("Undo the last change to the game"),
+                   NULL);
+  toolBar->AddTool(wxID_REDO, 
+                   wxEmptyString,
+                   wxBitmap(redo_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Redo the undone action"),
+                   _("Redo the last undone change"),
+                   NULL);
+
+  toolBar->AddSeparator();
+
+  toolBar->AddTool(GBT_MENU_EDIT_NEWPLAYER, 
+                   wxEmptyString,
+                   wxBitmap(newplayer_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+                   _("Add a new player"),
+                   _("Add a new player to the game"),
+                   NULL);
   if (m_doc->IsTree()) {
-    toolBar->AddTool(GBT_MENU_VIEW_ZOOMIN, wxBitmap(zoomin_xpm), wxNullBitmap,
-		     false, -1, -1, 0, 
-		     _("Zoom in"), _("Increase magnification"));
-    toolBar->AddTool(GBT_MENU_VIEW_ZOOMOUT, wxBitmap(zoomout_xpm), wxNullBitmap,
-		     false, -1, -1, 0, 
-		     _("Zoom out"), _("Decrease magnification"));
-    toolBar->AddTool(GBT_MENU_VIEW_ZOOMFIT, wxBitmap(zoomfit_xpm), wxNullBitmap,
-		     false, -1, -1, 0, _("Fit to window"), 
-		     _("Set magnification to see entire tree"));
+    toolBar->AddTool(GBT_MENU_VIEW_ZOOMIN, 
+                     wxEmptyString,
+                     wxBitmap(zoomin_xpm), 
+                     wxNullBitmap,
+                     wxITEM_NORMAL, 
+                     _("Zoom in"),
+                     _("Increase magnification"),
+                     NULL);
+    toolBar->AddTool(GBT_MENU_VIEW_ZOOMOUT, 
+                     wxEmptyString,
+                     wxBitmap(zoomout_xpm), 
+                     wxNullBitmap,
+                     wxITEM_NORMAL, 
+                     _("Zoom out"),
+                     _("Decrease magnification"),
+                     NULL);
+    toolBar->AddTool(GBT_MENU_VIEW_ZOOMFIT, 
+                     wxEmptyString,
+                     wxBitmap(zoomfit_xpm), 
+                     wxNullBitmap,
+                     wxITEM_NORMAL, 
+                     _("Fit to window"),
+                     _("Set magnification to see entrie tree"),
+                     NULL);
   }
 
   toolBar->AddSeparator();
-  toolBar->AddTool(GBT_MENU_FORMAT_DECIMALS_ADD, wxBitmap(adddecimal_xpm),
-		   wxNullBitmap, false, -1, -1, 0, 
-		   _("Increase the number of decimals displayed"),
-		   _("Increase the number of decimal places shown"));
 
-  toolBar->AddTool(GBT_MENU_FORMAT_DECIMALS_DELETE, wxBitmap(deldecimal_xpm),
-		   wxNullBitmap, false, -1, -1, 0, 
-		   _("Decrease the number of decimals displayed"),
-		   _("Decrease the number of decimal places shown"));
+  toolBar->AddTool(GBT_MENU_FORMAT_DECIMALS_ADD, 
+                   wxEmptyString,
+                   wxBitmap(adddecimal_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+		               _("Increase the number of decimals displayed"),
+		               _("Increase the number of decimal places shown"),
+                   NULL);
+  toolBar->AddTool(GBT_MENU_FORMAT_DECIMALS_DELETE,
+                   wxEmptyString,
+                   wxBitmap(deldecimal_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+		               _("Decrease the number of decimals displayed"),
+		               _("Decrease the number of decimal places shown"),
+                   NULL);
 
   toolBar->AddSeparator();
+  
   if (m_doc->IsTree()) {
-    toolBar->AddTool(GBT_MENU_VIEW_STRATEGIC, wxBitmap(table_xpm),
-		     wxNullBitmap, true, -1, -1, 0, 
-		     _("Display the reduced strategic representation "
-		       "of the game"),
-		     _("Display the reduced strategic representation "
-		       "of the game"));
+    toolBar->AddTool(GBT_MENU_VIEW_STRATEGIC,
+                     wxEmptyString,
+                     wxBitmap(table_xpm), 
+                     wxNullBitmap,
+                     wxITEM_CHECK, 
+		                 _("Display the reduced strategic representation of the game"),
+		                 _("Display the reduced strategic representation of the game"),
+                     NULL);
   }
-  toolBar->AddTool(GBT_MENU_VIEW_PROFILES, wxBitmap(profiles_xpm),
-		   wxNullBitmap, true, -1, -1, 0, 
-		   _("View the list of computed strategy profiles"), 
-		   _("Show or hide the list of computed strategy profiles"));
-  toolBar->AddTool(GBT_MENU_TOOLS_EQUILIBRIUM, wxBitmap(calc_xpm),
-		   wxNullBitmap, false, -1, -1, 0, 
-		   _("Compute Nash equilibria of this game"),
-		   _("Compute Nash equilibria of this game"));
+  toolBar->AddTool(GBT_MENU_VIEW_PROFILES,
+                   wxEmptyString,
+                   wxBitmap(profiles_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+		               _("View the list of computed strategy profiles"), 
+		               _("Show or hide the list of computed strategy profiles"),
+                   NULL);
+  toolBar->AddTool(GBT_MENU_TOOLS_EQUILIBRIUM,
+                   wxEmptyString,
+                   wxBitmap(calc_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+		               _("Compute Nash equilibria of this game"),
+		               _("Compute Nash equilibria of this game"),
+                   NULL);
   
   toolBar->AddSeparator();
-  toolBar->AddTool(wxID_ABOUT, wxBitmap(about_xpm), wxNullBitmap, false,
-		   -1, -1, 0, _("About Gambit"), _("About Gambit"));
+
+  toolBar->AddTool(wxID_ABOUT,
+                   wxEmptyString,
+                   wxBitmap(about_xpm), 
+                   wxNullBitmap,
+                   wxITEM_NORMAL, 
+		               _("About Gambit"),
+		               _("About Gambit"),
+                   NULL);
 
   toolBar->Realize();
   toolBar->SetRows(1);
@@ -690,7 +780,7 @@ void gbtGameFrame::MakeToolbar(void)
 
 void gbtGameFrame::OnFileNewEfg(wxCommandEvent &)
 {
-  Gambit::Game efg = new Gambit::GameRep;
+  Gambit::Game efg = Gambit::NewTree();
   efg->SetTitle("Untitled Extensive Game");
   efg->NewPlayer()->SetLabel("Player 1");
   efg->NewPlayer()->SetLabel("Player 2");
@@ -703,7 +793,7 @@ void gbtGameFrame::OnFileNewNfg(wxCommandEvent &)
   Gambit::Array<int> dim(2);
   dim[1] = 2;
   dim[2] = 2;
-  Gambit::Game nfg = new Gambit::GameRep(dim);
+  Gambit::Game nfg = Gambit::NewTable(dim);
   nfg->SetTitle("Untitled Strategic Game");
   nfg->GetPlayer(1)->SetLabel("Player 1");
   nfg->GetPlayer(2)->SetLabel("Player 2");
@@ -715,17 +805,14 @@ void gbtGameFrame::OnFileOpen(wxCommandEvent &)
 {
   wxFileDialog dialog(this, _("Choose file to open"), 
 		      wxGetApp().GetCurrentDir(), _T(""), 
-		      _("Gambit workbooks (*.gbt)|*.gbt|"
-			"Gambit extensive games (*.efg)|*.efg|"
-			"Gambit strategic games (*.nfg)|*.nfg|"
-			"All files (*.*)|*.*"));
+		      wxT("Gambit workbooks (*.gbt)|*.gbt|")
+			wxT("Gambit extensive games (*.efg)|*.efg|")
+			wxT("Gambit strategic games (*.nfg)|*.nfg|")
+			wxT("All files (*.*)|*.*"));
 
   if (dialog.ShowModal() == wxID_OK) {
     wxString filename = dialog.GetPath();
-
     wxGetApp().SetCurrentDir(wxPathOnly(filename));
-    wxConfig config(_T("Gambit"));
-    config.Write(_T("/General/CurrentDirectory"), wxPathOnly(filename));
 
     gbtAppLoadResult result = wxGetApp().LoadFile(filename);
     if (result == GBT_APP_OPEN_FAILED) {
@@ -759,22 +846,27 @@ void gbtGameFrame::OnFileSave(wxCommandEvent &p_event)
     wxFileDialog dialog(this, _("Choose file"),
 			wxPathOnly(m_doc->GetFilename()),
 			wxFileNameFromPath(m_doc->GetFilename()),
-			wxT("Gambit workbooks (*.gbt)|*.gbt|"
-			    "All files (*.*)|*.*"),
-			wxSAVE | wxOVERWRITE_PROMPT);
+			wxT("Gambit workbooks (*.gbt)|*.gbt|")
+			wxT("All files (*.*)|*.*"),
+			wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
     if (dialog.ShowModal() == wxID_OK) {
-      m_doc->SetFilename(dialog.GetPath());
+      try {
+	m_doc->DoSave(dialog.GetPath());
+      }
+      catch (std::exception &ex) {
+	gbtExceptionDialog(this, ex.what()).ShowModal();
+      }
     }
-    else {
-      return;
+  }
+  else {
+    try {
+      m_doc->DoSave(m_doc->GetFilename());
+    }
+    catch (std::exception &ex) {
+      gbtExceptionDialog(this, ex.what()).ShowModal();
     }
   }
-
-  std::ofstream file((const char *) m_doc->GetFilename().mb_str());
-  m_doc->SaveDocument(file);
-  m_doc->SetModified(false);
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_NONE);
 }
 
 void gbtGameFrame::OnFilePageSetup(wxCommandEvent &)
@@ -845,14 +937,12 @@ void gbtGameFrame::OnFileExportEfg(wxCommandEvent &)
 {
   wxFileDialog dialog(this, _("Choose file"), 
 		      wxGetApp().GetCurrentDir(), _T(""),
-		      _T("Gambit extensive games (*.efg)|*.efg|"
-			 "All files (*.*)|*.*"), 
-		      wxSAVE | wxOVERWRITE_PROMPT);
+		      wxT("Gambit extensive games (*.efg)|*.efg|")
+			 wxT("All files (*.*)|*.*"), 
+		      wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
   if (dialog.ShowModal() == wxID_OK) {
-    std::ofstream file((const char *) dialog.GetPath().mb_str());
-    m_doc->GetGame()->WriteEfgFile(file);
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_NONE);
+    m_doc->DoExportEfg(dialog.GetPath());
   }
 }
 
@@ -860,15 +950,12 @@ void gbtGameFrame::OnFileExportNfg(wxCommandEvent &)
 {
   wxFileDialog dialog(this, _("Choose file"), 
 		      wxGetApp().GetCurrentDir(), _T(""),
-		      _T("Gambit strategic games (*.nfg)|*.nfg|"
-			 "All files (*.*)|*.*"),
-		      wxSAVE | wxOVERWRITE_PROMPT);
+		      wxT("Gambit strategic games (*.nfg)|*.nfg|")
+			 wxT("All files (*.*)|*.*"),
+		      wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
   if (dialog.ShowModal() == wxID_OK) {
-    std::ofstream file((const char *) dialog.GetPath().mb_str());
-    m_doc->BuildNfg();
-    m_doc->GetGame()->WriteNfgFile(file);
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_NONE);
+    m_doc->DoExportNfg(dialog.GetPath());
   }
 }
 
@@ -896,8 +983,8 @@ void gbtGameFrame::OnFileExportGraphic(wxCommandEvent &p_event)
     filter = wxT("Windows bitmap files (*.bmp)|*.bmp") + filter;
     break;
   case GBT_MENU_FILE_EXPORT_JPEG:
-    filter = wxT("JPEG files (*.jpeg)|*.jpeg|"
-		 "JPG files (*.jpg)|*.jpg") + filter;
+    filter = wxT("JPEG files (*.jpeg)|*.jpeg|")
+		 wxT("JPG files (*.jpg)|*.jpg") + filter;
     break;
   case GBT_MENU_FILE_EXPORT_PNG:
     filter = wxT("PNG files (*.png)|*.png") + filter;
@@ -908,7 +995,7 @@ void gbtGameFrame::OnFileExportGraphic(wxCommandEvent &p_event)
 
   wxFileDialog dialog(this, _("Choose output file"), 
 		      wxGetApp().GetCurrentDir(), _T(""), filter,
-		      wxSAVE | wxOVERWRITE_PROMPT);
+		      wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
   if (dialog.ShowModal() == wxID_OK) {
     wxBitmapType code;
@@ -916,7 +1003,8 @@ void gbtGameFrame::OnFileExportGraphic(wxCommandEvent &p_event)
     case GBT_MENU_FILE_EXPORT_BMP:  code = wxBITMAP_TYPE_BMP;  break;
     case GBT_MENU_FILE_EXPORT_JPEG: code = wxBITMAP_TYPE_JPEG; break;
     case GBT_MENU_FILE_EXPORT_PNG:  code = wxBITMAP_TYPE_PNG;  break;
-    default: break;
+    default:   return;
+
     }
 
     if (!bitmap.SaveFile(dialog.GetPath(), code)) {
@@ -934,7 +1022,7 @@ void gbtGameFrame::OnFileExportPS(wxCommandEvent &)
   wxFileDialog dialog(this, _("Choose output file"),
 		      wxGetApp().GetCurrentDir(), _T(""),
 		      _T("PostScript files (*.ps)|*.ps"), 
-		      wxSAVE | wxOVERWRITE_PROMPT);
+		      wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
   if (dialog.ShowModal() == wxID_OK) {
     printData.SetFilename(dialog.GetPath());
@@ -968,9 +1056,9 @@ void gbtGameFrame::OnFileExportSVG(wxCommandEvent &)
 {
   wxFileDialog dialog(this, _("Choose output file"), 
 		      wxGetApp().GetCurrentDir(), _T(""),
-		      wxT("SVG files (*.svg)|*.svg|"
-			  "All files (*.*)|*.*"),
-		      wxSAVE | wxOVERWRITE_PROMPT);
+		      wxT("SVG files (*.svg)|*.svg|")
+			  wxT("All files (*.*)|*.*"),
+		      wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
   if (dialog.ShowModal() == wxID_OK) {
     if (m_efgPanel && m_efgPanel->IsShown()) {
@@ -985,9 +1073,9 @@ void gbtGameFrame::OnFileExportSVG(wxCommandEvent &)
 void gbtGameFrame::OnFileExit(wxCommandEvent &p_event)
 {
   if (wxGetApp().AreDocumentsModified()) {
-    if (wxMessageBox(_("There are modified games.\n"
-		       "Any unsaved changes will be lost!\n"
-		       "Close anyway?"), 
+    if (wxMessageBox(wxT("There are modified games.\n")
+		       wxT("Any unsaved changes will be lost!\n")
+		       wxT("Close anyway?"), 
 		     _("Warning"),
 		     wxOK | wxCANCEL) == wxCANCEL) {
       return;
@@ -1039,59 +1127,60 @@ void gbtGameFrame::OnEditRedo(wxCommandEvent &)
 void gbtGameFrame::OnEditInsertMove(wxCommandEvent &)
 { 
   gbtInsertMoveDialog dialog(this, m_doc);
-
   if (dialog.ShowModal() == wxID_OK)  {
     try {
-      if (!dialog.GetInfoset()) {
-	Gambit::GameInfoset infoset = 
-	  m_doc->GetSelectNode()->InsertMove(dialog.GetPlayer(),
-					     dialog.GetActions());
-	for (int act = 1; act <= infoset->NumActions(); act++) {
-	  infoset->GetAction(act)->SetLabel(Gambit::ToText(act));
-	}
+      if (dialog.GetInfoset()) {
+	m_doc->DoInsertMove(m_doc->GetSelectNode(), dialog.GetInfoset());
       }
       else {
-	m_doc->GetSelectNode()->InsertMove(dialog.GetInfoset());
+	m_doc->DoInsertMove(m_doc->GetSelectNode(), 
+			    dialog.GetPlayer(), dialog.GetActions());
       }
-      m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
     }
-    catch (Gambit::Exception &ex) {
-      gbtExceptionDialog(ex.GetDescription(), this);
+    catch (std::exception &ex) {
+      gbtExceptionDialog(this, ex.what()).ShowModal();
     }
   }
 }
 
 void gbtGameFrame::OnEditInsertAction(wxCommandEvent &)
 {
-  Gambit::GameNode node = m_doc->GetSelectNode();
-  if (!node || !node->GetInfoset())  return;
-
-  Gambit::GameAction action = node->GetInfoset()->InsertAction();
-  action->SetLabel(Gambit::ToText(action->GetNumber()));
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
+  try {
+    m_doc->DoInsertAction(m_doc->GetSelectNode());
+  }
+  catch (std::exception &ex) {
+    gbtExceptionDialog(this, ex.what()).ShowModal();
+  }
 }
 
 void gbtGameFrame::OnEditDeleteTree(wxCommandEvent &)
 {
-  if (!m_doc->GetSelectNode())  return;
-
-  m_doc->GetSelectNode()->DeleteTree();
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
+  try {
+    m_doc->DoDeleteTree(m_doc->GetSelectNode());
+  }
+  catch (std::exception &ex) {
+    gbtExceptionDialog(this, ex.what()).ShowModal();
+  }
 }
 
 void gbtGameFrame::OnEditDeleteParent(wxCommandEvent &)
 {
-  if (!m_doc->GetSelectNode() || !m_doc->GetSelectNode()->GetParent())  return;
-
-  m_doc->GetSelectNode()->DeleteParent();
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
+  try {
+    m_doc->DoDeleteParent(m_doc->GetSelectNode());
+  }
+  catch (std::exception &ex) {
+    gbtExceptionDialog(this, ex.what()).ShowModal();
+  }
 }
 
 void gbtGameFrame::OnEditRemoveOutcome(wxCommandEvent &)
 {
-  if (!m_doc->GetSelectNode() || !m_doc->GetSelectNode()->GetOutcome()) return;
-  m_doc->GetSelectNode()->SetOutcome(0);
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
+  try {
+    m_doc->DoRemoveOutcome(m_doc->GetSelectNode());
+  }
+  catch (std::exception &ex) {
+    gbtExceptionDialog(this, ex.what()).ShowModal();
+  }
 }
 
 void gbtGameFrame::OnEditReveal(wxCommandEvent &)
@@ -1101,12 +1190,12 @@ void gbtGameFrame::OnEditReveal(wxCommandEvent &)
   if (dialog.ShowModal() == wxID_OK) {
     try {
       for (int pl = 1; pl <= dialog.GetPlayers().Length(); pl++) {
-	m_doc->GetSelectNode()->GetInfoset()->Reveal(dialog.GetPlayers()[pl]);
+	m_doc->DoRevealAction(m_doc->GetSelectNode()->GetInfoset(),
+			      dialog.GetPlayers()[pl]);
       }
-      m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
     }
-    catch (Gambit::Exception &ex) {
-      gbtExceptionDialog(ex.GetDescription(), this);
+    catch (std::exception &ex) {
+      gbtExceptionDialog(this, ex.what()).ShowModal();
     }
   }
 }
@@ -1115,24 +1204,29 @@ void gbtGameFrame::OnEditNode(wxCommandEvent &)
 {
   dialogEditNode dialog(this, m_doc->GetSelectNode());
   if (dialog.ShowModal() == wxID_OK) {
-    m_doc->GetSelectNode()->SetLabel((const char *) dialog.GetNodeName().mb_str());
-    if (dialog.GetOutcome() > 0) {
-      m_doc->GetSelectNode()->SetOutcome(m_doc->GetGame()->GetOutcome(dialog.GetOutcome()));
-    }
-    else {
-      m_doc->GetSelectNode()->SetOutcome(0);
-    }
-
-    if (m_doc->GetSelectNode()->NumChildren() > 0 &&
-	dialog.GetInfoset() != m_doc->GetSelectNode()->GetInfoset()) {
-      if (dialog.GetInfoset() == 0) {
-	m_doc->GetSelectNode()->LeaveInfoset();
+    try {
+      m_doc->DoSetNodeLabel(m_doc->GetSelectNode(), dialog.GetNodeName());
+      if (dialog.GetOutcome() > 0) {
+	m_doc->DoSetOutcome(m_doc->GetSelectNode(),
+			    m_doc->GetGame()->GetOutcome(dialog.GetOutcome()));
       }
       else {
-	m_doc->GetSelectNode()->SetInfoset(dialog.GetInfoset());	
+	m_doc->DoSetOutcome(m_doc->GetSelectNode(), 0);
+      }
+
+      if (m_doc->GetSelectNode()->NumChildren() > 0 &&
+	  dialog.GetInfoset() != m_doc->GetSelectNode()->GetInfoset()) {
+	if (dialog.GetInfoset() == 0) {
+	  m_doc->DoLeaveInfoset(m_doc->GetSelectNode());
+	}
+	else {
+	  m_doc->DoSetInfoset(m_doc->GetSelectNode(), dialog.GetInfoset());
+	}
       }
     }
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
+    catch (std::exception &ex) {
+      gbtExceptionDialog(this, ex.what()).ShowModal();
+    }
   }
 }
 
@@ -1143,44 +1237,46 @@ void gbtGameFrame::OnEditMove(wxCommandEvent &)
 
   gbtEditMoveDialog dialog(this, infoset);
   if (dialog.ShowModal() == wxID_OK) {
-    infoset->SetLabel((const char *) dialog.GetInfosetName().mb_str());
-    
-    if (!infoset->IsChanceInfoset() && 
-	dialog.GetPlayer() != infoset->GetPlayer()->GetNumber()) {
-      infoset->SetPlayer(m_doc->GetGame()->GetPlayer(dialog.GetPlayer()));
-    }
+    try {
+      m_doc->DoSetInfosetLabel(infoset, dialog.GetInfosetName());
 
-    for (int act = 1; act <= infoset->NumActions(); act++) {
-      infoset->GetAction(act)->SetLabel(dialog.GetActionName(act));
-      if (infoset->IsChanceInfoset()) {
-	infoset->SetActionProb(infoset->GetAction(act)->GetNumber(),
-			       dialog.GetActionProb(act));
+      if (!infoset->IsChanceInfoset() && 
+	  dialog.GetPlayer() != infoset->GetPlayer()->GetNumber()) {
+	m_doc->DoSetPlayer(infoset, 
+			   m_doc->GetGame()->GetPlayer(dialog.GetPlayer()));
+      }
+
+      for (int act = 1; act <= infoset->NumActions(); act++) {
+	m_doc->DoSetActionLabel(infoset->GetAction(act), 
+				dialog.GetActionName(act));
+	if (infoset->IsChanceInfoset()) {
+	  m_doc->DoSetActionProb(infoset, infoset->GetAction(act)->GetNumber(),
+				 dialog.GetActionProb(act));
+	}
       }
     }
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
+    catch (std::exception &ex) {
+      gbtExceptionDialog(this, ex.what()).ShowModal();
+    }
   }
 }
 
 void gbtGameFrame::OnEditGame(wxCommandEvent &)
 {
   gbtGamePropertiesDialog dialog(this, m_doc);
-
   if (dialog.ShowModal() == wxID_OK) {
-    Gambit::Game game = m_doc->GetGame();
-    game->SetTitle((const char *) dialog.GetTitle().mb_str());
-    game->SetComment((const char *) dialog.GetComment().mb_str());
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
+    m_doc->DoSetTitle(dialog.GetTitle(), dialog.GetComment());
   }
 }
 
 void gbtGameFrame::OnEditNewPlayer(wxCommandEvent &)
 {
-  Gambit::GamePlayer player = m_doc->GetGame()->NewPlayer();
-  player->SetLabel("Player " + Gambit::ToText(player->GetNumber()));
-  if (!m_doc->IsTree()) {
-    player->GetStrategy(1)->SetLabel("1");
+  try {
+    m_doc->DoNewPlayer();
+  }
+  catch (std::exception &ex) {
+    gbtExceptionDialog(this, ex.what()).ShowModal();
   }
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_GAME);
 }
 
 //----------------------------------------------------------------------
@@ -1215,16 +1311,30 @@ void gbtGameFrame::OnViewStrategic(wxCommandEvent &p_event)
 {
   if (m_efgPanel->IsShown()) {
     // We are switching to strategic view
-
     if (!m_doc->GetGame()->IsPerfectRecall()) {
-      if (wxMessageBox(_("This is not a game of perfect recall\n"
-			 "Do you wish to continue?"), 
-		       _("Strategic game"), 
+      wxMessageBox(wxT("This is not a game of perfect recall.\n")
+		   wxT("Computing the reduced strategic representation ")
+		   wxT("of this game is not supported."),
+		   wxT("Show strategic game"),
+		   wxOK);
+      return;
+    }
+    
+    int ncont = m_doc->GetGame()->NumStrategyContingencies();
+    if (!m_nfgPanel && ncont >= 50000) {
+      if (wxMessageBox(wxString::Format(wxT("This game has %d contingencies in strategic form.\n"), ncont) +
+		       wxT("Performance in browsing strategic form will be poor,\n") +
+		       wxT("and may render the program nonresponsive.\n") +
+		       wxT("Do you wish to continue?"),
+		       _("Large strategic game warning"),
 		       wxOK | wxCANCEL | wxALIGN_CENTER, this) != wxOK) {
 	return;
       }
     }
-    
+
+    if (!m_nfgPanel) {
+      m_nfgPanel = new gbtNfgPanel(m_splitter, m_doc);
+    }
     m_doc->BuildNfg();
 
     m_splitter->ReplaceWindow(m_efgPanel, m_nfgPanel);
@@ -1234,6 +1344,7 @@ void gbtGameFrame::OnViewStrategic(wxCommandEvent &p_event)
       m_analysisPanel->ShowMixed(true);
     }
     m_nfgPanel->SetFocus();
+    m_nfgPanel->OnUpdate();
   }
   else {
     m_splitter->ReplaceWindow(m_nfgPanel, m_efgPanel);
@@ -1253,8 +1364,6 @@ void gbtGameFrame::OnViewStrategic(wxCommandEvent &p_event)
   GetToolBar()->EnableTool(GBT_MENU_VIEW_ZOOMIN, !p_event.IsChecked());
   GetToolBar()->EnableTool(GBT_MENU_VIEW_ZOOMOUT, !p_event.IsChecked());
   GetToolBar()->EnableTool(GBT_MENU_VIEW_ZOOMFIT, !p_event.IsChecked());
-
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_NONE);
 }
 
 //----------------------------------------------------------------------
@@ -1263,13 +1372,12 @@ void gbtGameFrame::OnViewStrategic(wxCommandEvent &p_event)
 
 void gbtGameFrame::OnFormatLayout(wxCommandEvent &)
 {
-  gbtStyle &settings = m_doc->GetStyle();
-
-  gbtLayoutDialog dialog(this, settings);
+  gbtStyle style = m_doc->GetStyle();
+  gbtLayoutDialog dialog(this, style);
 
   if (dialog.ShowModal() == wxID_OK) {
-    dialog.GetSettings(settings);
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_VIEWS);
+    dialog.GetSettings(style);
+    m_doc->SetStyle(style);
   }
 }
 
@@ -1278,11 +1386,12 @@ void gbtGameFrame::OnFormatLabels(wxCommandEvent &)
   gbtLegendDialog dialog(this, m_doc->GetStyle());
 
   if (dialog.ShowModal() == wxID_OK) {
-    m_doc->GetStyle().SetNodeAboveLabel(dialog.GetNodeAbove());
-    m_doc->GetStyle().SetNodeBelowLabel(dialog.GetNodeBelow());
-    m_doc->GetStyle().SetBranchAboveLabel(dialog.GetBranchAbove());
-    m_doc->GetStyle().SetBranchBelowLabel(dialog.GetBranchBelow());
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_VIEWS);
+    gbtStyle style = m_doc->GetStyle();
+    style.SetNodeAboveLabel(dialog.GetNodeAbove());
+    style.SetNodeBelowLabel(dialog.GetNodeBelow());
+    style.SetBranchAboveLabel(dialog.GetBranchAbove());
+    style.SetBranchBelowLabel(dialog.GetBranchBelow());
+    m_doc->SetStyle(style);
   }
 }
 
@@ -1293,21 +1402,24 @@ void gbtGameFrame::OnFormatFonts(wxCommandEvent &)
   wxFontDialog dialog(this, data);
   
   if (dialog.ShowModal() == wxID_OK) {
-    m_doc->GetStyle().SetFont(dialog.GetFontData().GetChosenFont());
-    m_doc->UpdateViews(GBT_DOC_MODIFIED_VIEWS);
+    gbtStyle style = m_doc->GetStyle();
+    style.SetFont(dialog.GetFontData().GetChosenFont());
+    m_doc->SetStyle(style);
   }
 }
 
 void gbtGameFrame::OnFormatDecimalsAdd(wxCommandEvent &)
 {
-  m_doc->GetStyle().SetNumDecimals(m_doc->GetStyle().NumDecimals()+1);
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_VIEWS);
+  gbtStyle style = m_doc->GetStyle();
+  style.SetNumDecimals(style.NumDecimals()+1);
+  m_doc->SetStyle(style);
 }
 
 void gbtGameFrame::OnFormatDecimalsDelete(wxCommandEvent &)
 {
-  m_doc->GetStyle().SetNumDecimals(m_doc->GetStyle().NumDecimals()-1);
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_VIEWS);
+  gbtStyle style = m_doc->GetStyle();
+  style.SetNumDecimals(style.NumDecimals()-1);
+  m_doc->SetStyle(style);
 }
 
 //----------------------------------------------------------------------
@@ -1326,9 +1438,32 @@ void gbtGameFrame::OnToolsDominance(wxCommandEvent &p_event)
 
 void gbtGameFrame::OnToolsEquilibrium(wxCommandEvent &)
 {
+  if (!m_doc->GetGame()->IsPerfectRecall()) {
+    wxMessageBox(wxT("This is not a game of perfect recall.\n")
+		 wxT("Computing Nash equilibria of this game ")
+		 wxT("is not supported."),
+		 wxT("Show strategic game"),
+		 wxOK);
+    return;
+  }
+  
   gbtNashChoiceDialog dialog(this, m_doc);
 
   if (dialog.ShowModal() == wxID_OK) {
+    if (dialog.UseStrategic()) {
+      int ncont = m_doc->GetGame()->NumStrategyContingencies();
+      if (ncont >= 50000) {
+	if (wxMessageBox(wxString::Format(wxT("This game has %d contingencies in strategic form.\n"), ncont) +
+			 wxT("Performance in solving strategic form will be poor,\n") +
+			 wxT("and may render the program nonresponsive.\n") +
+			 wxT("Do you wish to continue?"),
+			 _("Large strategic game warning"),
+			 wxOK | wxCANCEL | wxALIGN_CENTER, this) != wxOK) {
+	  return;
+	}
+      }
+    }
+
     gbtAnalysisOutput *command = dialog.GetCommand();
 
     gbtNashMonitorDialog dialog(this, m_doc, command);
@@ -1346,14 +1481,21 @@ void gbtGameFrame::OnToolsEquilibrium(wxCommandEvent &)
       }
     }
   }
-
-  m_doc->UpdateViews(GBT_DOC_MODIFIED_VIEWS);
 }
 
 extern void LogitStrategic(wxWindow *, gbtGameDocument *);
 
 void gbtGameFrame::OnToolsQre(wxCommandEvent &)
 {
+  if (!m_doc->GetGame()->IsPerfectRecall()) {
+    wxMessageBox(wxT("This is not a game of perfect recall.\n")
+		 wxT("Computing quantal response equilibria of this game ")
+		 wxT("is not supported."),
+		 wxT("Show strategic game"),
+		 wxOK);
+    return;
+  }
+
   if (m_efgPanel && m_splitter->GetWindow1() == m_efgPanel) {
     gbtLogitBehavDialog(this, m_doc).ShowModal();
   }
@@ -1384,9 +1526,9 @@ void gbtGameFrame::OnUnsplit(wxSplitterEvent &)
 void gbtGameFrame::OnCloseWindow(wxCloseEvent &p_event)
 {
   if (p_event.CanVeto() && m_doc->IsModified()) {
-    if (wxMessageBox(_("Game has been modified.\n"
-		       "Unsaved changes will be lost!\n"
-		       "Close anyway?"), 
+    if (wxMessageBox(wxT("Game has been modified.\n")
+		       wxT("Unsaved changes will be lost!\n")
+		       wxT("Close anyway?"), 
 		     _("Warning"),
 		     wxOK | wxCANCEL) == wxCANCEL) {
       p_event.Veto();
