@@ -12,10 +12,6 @@
 #ifndef __WX_SHEETDEF_H__
 #define __WX_SHEETDEF_H__
 
-//#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-//    #pragma interface "sheetdef.h"
-//#endif
-
 #include "wx/defs.h"
 #include "wx/datetime.h"
 #include "wx/sheet/wx24defs.h"   // wx2.4 backwards compatibility
@@ -187,7 +183,7 @@ enum wxSheetSelectionMode_Type
                                   // direct calls to the selections work however
     wxSHEET_SelectCells = 0x0002, // single cells, blocks, rows, and cols
     wxSHEET_SelectRows  = 0x0004, // only rows can be selected
-    wxSHEET_SelectCols  = 0x0008, // only cols can be selected
+    wxSHEET_SelectCols  = 0x0008 // only cols can be selected
 };
 
 // ----------------------------------------------------------------------------
