@@ -9,10 +9,6 @@
 // Licence:     wxWidgets licence
 ///////////////////////////////////////////////////////////////////////////////
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "sheetedt.h"
-#endif
-
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
@@ -39,7 +35,7 @@
 #include "wx/tokenzr.h"
 #include "wx/spinctrl.h"
 // Required for wxIs... functions
-#include <ctype.h>
+#include <cctype>
 
 #define PRINT_RECT(s, b) wxPrintf(wxT("%s %d %d %d %d - w%d h%d\n"), wxT(s), b.GetTop(), b.GetLeft(), b.GetBottom(), b.GetRight(), b.GetWidth(), b.GetHeight());
 
@@ -1391,7 +1387,7 @@ void wxSheetCellChoiceEditorRefData::BeginEdit(const wxSheetCoords& coords, wxSh
         int pos = GetComboBox()->FindString(m_startValue);
         if (pos == -1)
             pos = 0;
-        if (GetComboBox()->GetCount() > pos)
+        if (static_cast<int>(GetComboBox()->GetCount()) > pos)
             GetComboBox()->SetSelection(pos);
     }
     GetComboBox()->SetInsertionPointEnd();
