@@ -1,6 +1,6 @@
 //
 // This file is part of Gambit
-// Copyright (c) 1994-2010, The Gambit Project (http://www.gambit-project.org)
+// Copyright (c) 1994-2014, The Gambit Project (http://www.gambit-project.org)
 //
 // FILE: src/libgambit/file.cc
 // Parser for reading game savefiles
@@ -20,69 +20,14 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
 
-#include <stdlib.h>
-#include <ctype.h>
+#include <cstdlib>
+#include <cctype>
 #include <iostream>
 #include <sstream>
-
+#include <map>
 
 #include "libgambit.h"
 
-namespace Gambit {
-
-
-/// This extension to the game representation classes allows for efficient
-/// mass creation of outcomes.  For large strategic games, sequentially
-/// creating outcomes as you find them (in the outcome version of the
-/// .nfg format) is brutally inefficient.  For one 21x21x21x21 game,
-/// the naive code took about 97s to read the game; with this version,
-/// it takes about 5s on the same system.
-class TableFileGameRep : public GameRep {
-public:
-  /// @name Lifecycle
-  //@{
-  /// Construct a new table game with the given dimension, plus enough
-  /// outcomes to fill the table
-  TableFileGameRep(const Array<int> &p_dim);
-  //@}
-
-  /// @name Extensions
-  //@{
-  /// Delete all outcomes numbered above 'p_nOutcomes'
-  void TruncateOutcomes(int p_nOutcomes);
-  //@}
-};
-
-TableFileGameRep::TableFileGameRep(const Array<int> &p_dim)
-  : GameRep(p_dim)
-{
-  m_outcomes = Array<GameOutcomeRep *>(m_results.Length());
-  for (int i = 1; i <= m_outcomes.Length(); i++) {
-    m_outcomes[i] = new GameOutcomeRep(this, i);
-  }
-}
-
-void TableFileGameRep::TruncateOutcomes(int p_nOutcomes)
-{
-  if (p_nOutcomes >= m_outcomes.Length()) {
-    return;
-  }
-
-  Array<GameOutcomeRep *> outcomes(m_outcomes);
-
-  m_outcomes = Array<GameOutcomeRep *>(p_nOutcomes);
-  for (int i = 1; i <= p_nOutcomes; i++) {
-    m_outcomes[i] = outcomes[i];
-  }
-
-  // We just use 'delete' here, instead of Invalidate(), because this
-  // code is only intended to be called in the file reader, and therefore
-  // there should be no other references to these outcomes elsewhere
-  for (int i = p_nOutcomes + 1; i <= outcomes.Length(); delete outcomes[i++]);
-}
-
-}
-
 namespace {
 // This anonymous namespace encapsulates the file-parsing code
 
@@ -103,33 +48,58 @@ private:
   std::istream &m_file;
 
   int m_currentLine;
+  int m_currentColumn;
   GameFileToken m_lastToken;
   std::string m_lastText;
 
+  void ReadChar(char& c);
+  void UnreadChar(void);
+  void IncreaseLine(void);
+
 public:
-  GameParserState(std::istream &p_file) : 
-    m_file(p_file), m_currentLine(1) { }
+  GameParserState(std::istream &p_file) :
+    m_file(p_file), m_currentLine(1), m_currentColumn(1) { }
 
   GameFileToken GetNextToken(void);
   GameFileToken GetCurrentToken(void) const { return m_lastToken; }
   int GetCurrentLine(void) const { return m_currentLine; }
+  int GetCurrentColumn(void) const { return m_currentColumn; }
+  std::string CreateLineMsg(const std::string &msg);
   const std::string &GetLastText(void) const { return m_lastText; }
-};  
+};
+
+void GameParserState::ReadChar(char& c)
+{
+  m_file.get(c);
+  m_currentColumn++;
+}
+
+void GameParserState::UnreadChar(void)
+{
+  m_file.unget();
+  m_currentColumn--;
+}
+
+void GameParserState::IncreaseLine(void){
+  m_currentLine++;
+  // Reset column
+  m_currentColumn = 1;
+}
 
 GameFileToken GameParserState::GetNextToken(void)
 {
-  char c = ' '; 
+  char c = ' ';
   if (m_file.eof()) {
     return (m_lastToken = TOKEN_EOF);
   }
 
   while (isspace(c)) {
-    m_file.get(c);
+    ReadChar(c);
     if (!m_file.good()) {
       return (m_lastToken = TOKEN_EOF);
-    } 
+    }
     else if (c == '\n') {
-      m_currentLine++;
+      IncreaseLine();
     }
   }
 
@@ -145,74 +115,74 @@ GameFileToken GameParserState::GetNextToken(void)
   else if (isdigit(c) || c == '-' || c == '+') {
     std::string buf;
     buf += c;
-    m_file.get(c);
+    ReadChar(c);
 
     while (!m_file.eof() && m_file.good() && isdigit(c)) {
       buf += c;
-      m_file.get(c);
+      ReadChar(c);
     }
-    
+
     if (m_file.eof() || !m_file.good()) {
       m_lastText = buf;
       return (m_lastToken = TOKEN_NUMBER);
-    }      
+    }
 
     if (c == '.') {
       buf += c;
-      m_file.get(c);
+      ReadChar(c);
       while (isdigit(c)) {
-      	buf += c;
-      	m_file.get(c);
+        buf += c;
+        ReadChar(c);
       }
 
       if (c == 'e' || c == 'E') {
-	buf += c;
-	m_file.get(c);
-	if (c == '+' && c == '-' && !isdigit(c)) {
-	  throw InvalidFileException();
-	}
-	buf += c;
-	m_file.get(c);
-	while (isdigit(c)) {
-	  buf += c;
-	  m_file.get(c);
-	}
+        buf += c;
+        ReadChar(c);
+        if (c == '+' && c == '-' && !isdigit(c)) {
+          throw InvalidFileException(CreateLineMsg("Invalid Token +/-"));
+        }
+        buf += c;
+        ReadChar(c);
+        while (isdigit(c)) {
+          buf += c;
+          ReadChar(c);
+        }
       }
 
-      m_file.unget();
+      UnreadChar();
       m_lastText = buf;
 
       return (m_lastToken = TOKEN_NUMBER);
     }
     else if (c == '/') {
       buf += c;
-      m_file.get(c);
+      ReadChar(c);
       while (isdigit(c)) {
-      	buf += c;
-      	m_file.get(c);
+        buf += c;
+        ReadChar(c);
       }
-      m_file.unget();
+      UnreadChar();
       m_lastText = buf;
       return (m_lastToken = TOKEN_NUMBER);
     }
     else if (c == 'e' || c == 'E') {
       buf += c;
-      m_file.get(c);
+      ReadChar(c);
       if (c == '+' && c == '-' && !isdigit(c)) {
-	throw InvalidFileException();
+        throw InvalidFileException(CreateLineMsg("Invalid Token +/-"));
       }
       buf += c;
-      m_file.get(c);
+      ReadChar(c);
       while (isdigit(c)) {
-	buf += c;
-	m_file.get(c);
+        buf += c;
+        ReadChar(c);
       }
-      m_file.unget();
+      UnreadChar();
       m_lastText = buf;
       return (m_lastToken = TOKEN_NUMBER);
     }
     else {
-      m_file.unget();
+      UnreadChar();
       m_lastText = buf;
       return (m_lastToken = TOKEN_NUMBER);
     }
@@ -220,13 +190,13 @@ GameFileToken GameParserState::GetNextToken(void)
   else if (c == '.') {
     std::string buf;
     buf += c;
-    m_file.get(c);
-    
+    ReadChar(c);
+
     while (isdigit(c)) {
       buf += c;
-      m_file.get(c);
+      ReadChar(c);
     }
-    m_file.unget();
+    UnreadChar();
     m_lastText = buf;
     return (m_lastToken = TOKEN_NUMBER);
   }
@@ -234,42 +204,49 @@ GameFileToken GameParserState::GetNextToken(void)
   else if (c == '"') {
     // We need to do a little magic here, since escaped quotes inside
     // the string are treated as quotes (not end-of-string)
-    m_file.unget();
+    UnreadChar();
     char a;
-  
+
     m_lastText = "";
-  
+
     do  {
-      m_file.get(a);
-      if (isspace(a) && a=='\n') {
-        m_currentLine++;
+      ReadChar(a);
+      if (a == '\n') {
+        IncreaseLine();
       }
-    }  while (isspace(a));
+    } while (isspace(a));
 
     if (a == '\"')  {
       bool lastslash = false;
 
-      m_file.get(a);
+      ReadChar(a);
       while  (a != '\"' || lastslash)  {
-      	if (lastslash && a == '"')  
-      	  m_lastText += '"';
-      	else if (lastslash)  {
-      	  m_lastText += '\\';
-      	  m_lastText += a;
-      	}
-      	else if (a != '\\')
-      	  m_lastText += a;
-      	
-      	lastslash = (a == '\\');
-      	m_file.get(a);
+	if (m_file.eof() || !m_file.good())  {
+	  throw InvalidFileException(CreateLineMsg("End of file encountered when reading string label"));
+	}
+        if (lastslash && a == '"') {
+          m_lastText += '"';
+	}
+        else if (lastslash)  {
+          m_lastText += '\\';
+          m_lastText += a;
+        }
+        else if (a != '\\')
+          m_lastText += a;
+
+        lastslash = (a == '\\');
+        ReadChar(a);
       }
     }
     else  {
       do  {
       	m_lastText += a;
-      	m_file.get(a);
-        if (isspace(a) && a=='\n') {
-          m_currentLine++;
+        ReadChar(a);
+	if (m_file.eof() || !m_file.good())  {
+	  throw InvalidFileException(CreateLineMsg("End of file encountered when reading string label"));
+	}
+        if (a == '\n') {
+          IncreaseLine();
         }
       }  while (!isspace(a));
     }
@@ -280,11 +257,17 @@ GameFileToken GameParserState::GetNextToken(void)
   m_lastText = "";
   while (!isspace(c) && !m_file.eof()) {
     m_lastText += c;
-    m_file.get(c);
+    ReadChar(c);
   }
   return (m_lastToken = TOKEN_SYMBOL);
 }
 
+std::string GameParserState::CreateLineMsg(const std::string &msg)
+{
+  std::stringstream stream;
+  stream << "line " << m_currentLine << ":" << m_currentColumn << ": " << msg;
+  return stream.str();
+}
 
 class TableFilePlayer {
 public:
@@ -351,7 +334,7 @@ int TableFileGame::NumStrategies(int p_player) const
 {
   TableFilePlayer *player = m_firstPlayer;
   int pl = 1;
-  
+
   while (player && pl < p_player) {
     player = player->m_next;
     pl++;
@@ -387,7 +370,7 @@ std::string TableFileGame::GetStrategy(int p_player, int p_strategy) const
 {
   TableFilePlayer *player = m_firstPlayer;
   int pl = 1;
-  
+
   while (player && pl < p_player) {
     player = player->m_next;
     pl++;
@@ -404,7 +387,8 @@ std::string TableFileGame::GetStrategy(int p_player, int p_strategy) const
 void ReadPlayers(GameParserState &p_state, TableFileGame &p_data)
 {
   if (p_state.GetNextToken() != TOKEN_LBRACE) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Expecting '{' before players"));
   }
 
   while (p_state.GetNextToken() == TOKEN_TEXT) {
@@ -412,7 +396,8 @@ void ReadPlayers(GameParserState &p_state, TableFileGame &p_data)
   }
 
   if (p_state.GetCurrentToken() != TOKEN_RBRACE) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Expecting '}' after players"));
   }
 
   p_state.GetNextToken();
@@ -421,7 +406,8 @@ void ReadPlayers(GameParserState &p_state, TableFileGame &p_data)
 void ReadStrategies(GameParserState &p_state, TableFileGame &p_data)
 {
   if (p_state.GetCurrentToken() != TOKEN_LBRACE) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Expecting '{' before strategies"));
   }
   p_state.GetNextToken();
 
@@ -430,16 +416,17 @@ void ReadStrategies(GameParserState &p_state, TableFileGame &p_data)
 
     while (p_state.GetCurrentToken() == TOKEN_LBRACE) {
       if (!player) {
-	// Not enough players for number of strategy entries
-	throw InvalidFileException();
+        throw InvalidFileException(p_state.CreateLineMsg(
+          "Not enough players for number of strategy entries"));
       }
 
       while (p_state.GetNextToken() == TOKEN_TEXT) {
-	player->m_strategies.Append(p_state.GetLastText());
+        player->m_strategies.Append(p_state.GetLastText());
       }
 
       if (p_state.GetCurrentToken() != TOKEN_RBRACE) {
-	throw InvalidFileException();
+        throw InvalidFileException(
+          p_state.CreateLineMsg("Expecting '}' after player strategy"));
       }
 
       p_state.GetNextToken();
@@ -447,12 +434,13 @@ void ReadStrategies(GameParserState &p_state, TableFileGame &p_data)
     }
 
     if (player) {
-      // Players with strategies undefined
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Players with undefined strategies"));
     }
 
     if (p_state.GetCurrentToken() != TOKEN_RBRACE) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Expecting '}' after strategies"));
     }
 
     p_state.GetNextToken();
@@ -462,12 +450,12 @@ void ReadStrategies(GameParserState &p_state, TableFileGame &p_data)
 
     while (p_state.GetCurrentToken() == TOKEN_NUMBER) {
       if (!player) {
-	// Not enough players for number of strategy entries
-	throw InvalidFileException();
+        throw InvalidFileException(p_state.CreateLineMsg(
+          "Not enough players for number of strategy entries"));
       }
 
       for (int st = 1; st <= atoi(p_state.GetLastText().c_str()); st++) {
-	player->m_strategies.Append(ToText(Integer(st)));
+        player->m_strategies.Append(lexical_cast<std::string>(st));
       }
 
       p_state.GetNextToken();
@@ -475,18 +463,20 @@ void ReadStrategies(GameParserState &p_state, TableFileGame &p_data)
     }
 
     if (p_state.GetCurrentToken() != TOKEN_RBRACE) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Expecting '}' after strategies"));
     }
 
     if (player) {
-      // Players with strategies undefined
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Players with strategies undefined"));
     }
 
     p_state.GetNextToken();
   }
   else {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Unrecognizable strategies format"));
   }
 }
 
@@ -494,15 +484,18 @@ void ParseNfgHeader(GameParserState &p_state, TableFileGame &p_data)
 {
   if (p_state.GetNextToken() != TOKEN_NUMBER ||
       p_state.GetLastText() != "1") {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Accepting only NFG version 1"));
   }
 
-  if (p_state.GetNextToken() != TOKEN_SYMBOL || 
+  if (p_state.GetNextToken() != TOKEN_SYMBOL ||
       (p_state.GetLastText() != "D" && p_state.GetLastText() != "R")) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Accepting only NFG D or R data type"));
   }
   if (p_state.GetNextToken() != TOKEN_TEXT) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Game title missing"));
   }
   p_data.m_title = p_state.GetLastText();
 
@@ -517,7 +510,7 @@ void ParseNfgHeader(GameParserState &p_state, TableFileGame &p_data)
 }
 
 
-void ReadOutcomeList(GameParserState &p_parser, TableFileGameRep *p_nfg)
+void ReadOutcomeList(GameParserState &p_parser, GameRep *p_nfg)
 {
   if (p_parser.GetNextToken() == TOKEN_RBRACE) {
     // Special case: empty outcome list
@@ -526,7 +519,8 @@ void ReadOutcomeList(GameParserState &p_parser, TableFileGameRep *p_nfg)
   }
 
   if (p_parser.GetCurrentToken() != TOKEN_LBRACE) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_parser.CreateLineMsg("Expecting '{' before outcome"));
   }
 
   int nOutcomes = 0;
@@ -534,9 +528,10 @@ void ReadOutcomeList(GameParserState &p_parser, TableFileGameRep *p_nfg)
   while (p_parser.GetCurrentToken() == TOKEN_LBRACE) {
     nOutcomes++;
     int pl = 1;
-    
+
     if (p_parser.GetNextToken() != TOKEN_TEXT) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_parser.CreateLineMsg("Expecting string for outcome"));
     }
 
     GameOutcome outcome;
@@ -553,72 +548,69 @@ void ReadOutcomeList(GameParserState &p_parser, TableFileGameRep *p_nfg)
 
     try {
       while (p_parser.GetCurrentToken() == TOKEN_NUMBER) {
-	outcome->SetPayoff(pl++, p_parser.GetLastText());
-	if (p_parser.GetNextToken() == TOKEN_COMMA) {
-	  p_parser.GetNextToken();
-	}
+        outcome->SetPayoff(pl++, p_parser.GetLastText());
+        if (p_parser.GetNextToken() == TOKEN_COMMA) {
+            p_parser.GetNextToken();
+        }
       }
     }
     catch (IndexException &) {
       // This would be triggered by too many payoffs
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_parser.CreateLineMsg("Exceeded number of players in outcome"));
     }
 
     if (pl <= p_nfg->NumPlayers() ||
-	p_parser.GetCurrentToken() != TOKEN_RBRACE) {
-      throw InvalidFileException();
+     p_parser.GetCurrentToken() != TOKEN_RBRACE) {
+      throw InvalidFileException(
+        p_parser.CreateLineMsg("Insufficient number of players in outcome"));
     }
 
     p_parser.GetNextToken();
   }
 
-  p_nfg->TruncateOutcomes(nOutcomes);
-
   if (p_parser.GetCurrentToken() != TOKEN_RBRACE) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+        p_parser.CreateLineMsg("Expecting '}' after outcome"));
   }
   p_parser.GetNextToken();
 }
 
-void ParseOutcomeBody(GameParserState &p_parser, TableFileGameRep *p_nfg)
+void ParseOutcomeBody(GameParserState &p_parser, GameRep *p_nfg)
 {
   ReadOutcomeList(p_parser, p_nfg);
 
-  StrategyIterator iter(StrategySupport(static_cast<GameRep *>(p_nfg)));
+  StrategyProfileIterator iter(StrategySupportProfile(static_cast<GameRep *>(p_nfg)));
 
   while (p_parser.GetCurrentToken() != TOKEN_EOF) {
     if (p_parser.GetCurrentToken() != TOKEN_NUMBER) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_parser.CreateLineMsg("Expecting outcome index"));
     }
 
     int outcomeId = atoi(p_parser.GetLastText().c_str());
     if (outcomeId > 0)  {
-      iter->SetOutcome(p_nfg->GetOutcome(outcomeId));
+      (*iter)->SetOutcome(p_nfg->GetOutcome(outcomeId));
     }
     else {
-      iter->SetOutcome(0);
+      (*iter)->SetOutcome(0);
     }
     p_parser.GetNextToken();
     iter++;
   }
 }
 
-void ParsePayoffBody(GameParserState &p_parser, 
-			    TableFileGameRep *p_nfg)
+void ParsePayoffBody(GameParserState &p_parser, GameRep *p_nfg)
 {
-  StrategyIterator iter(StrategySupport(static_cast<GameRep *>(p_nfg)));
+  StrategyProfileIterator iter(StrategySupportProfile(static_cast<GameRep *>(p_nfg)));
   int pl = 1;
 
   while (p_parser.GetCurrentToken() != TOKEN_EOF) {
-    if (pl == 1) {
-      iter->SetOutcome(p_nfg->NewOutcome());
-    }
-
     if (p_parser.GetCurrentToken() == TOKEN_NUMBER) {
-      iter->GetOutcome()->SetPayoff(pl, p_parser.GetLastText());
+      (*iter)->GetOutcome()->SetPayoff(pl, p_parser.GetLastText());
     }
     else {
-      throw InvalidFileException();
+      throw InvalidFileException(p_parser.CreateLineMsg("Expecting payoff"));
     }
 
     if (++pl > p_nfg->NumPlayers()) {
@@ -636,21 +628,21 @@ Game BuildNfg(GameParserState &p_parser, TableFileGame &p_data)
     dim[pl] = p_data.NumStrategies(pl);
   }
 
-  TableFileGameRep *nfg = new TableFileGameRep(dim);
+  GameRep *nfg = NewTable(dim);
   // Assigning this to the container assures that, if something goes
   // wrong, the class will automatically be cleaned up
   Game game = nfg;
 
   nfg->SetTitle(p_data.m_title);
   nfg->SetComment(p_data.m_comment);
-  
+
   for (int pl = 1; pl <= dim.Length(); pl++) {
     nfg->GetPlayer(pl)->SetLabel(p_data.GetPlayer(pl));
     for (int st = 1; st <= dim[pl]; st++) {
       nfg->GetPlayer(pl)->GetStrategy(st)->SetLabel(p_data.GetStrategy(pl,st));
     }
   }
-  
+
   if (p_parser.GetCurrentToken() == TOKEN_LBRACE) {
     ParseOutcomeBody(p_parser, nfg);
   }
@@ -658,7 +650,8 @@ Game BuildNfg(GameParserState &p_parser, TableFileGame &p_data)
     ParsePayoffBody(p_parser, nfg);
   }
   else {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_parser.CreateLineMsg("Expecting outcome or payoff"));
   }
 
   return game;
@@ -670,33 +663,32 @@ Game BuildNfg(GameParserState &p_parser, TableFileGame &p_data)
 //                  Temporary representation classes
 //=========================================================================
 
-#include "map.h"
-
 class TreeData {
 public:
-  Map<int, GameOutcome> m_outcomeMap;
-  Map<int, GameInfoset> m_chanceInfosetMap;
-  List<Map<int, GameInfoset> > m_infosetMap;
+  std::map<int, GameOutcome> m_outcomeMap;
+  std::map<int, GameInfoset> m_chanceInfosetMap;
+  List<std::map<int, GameInfoset> > m_infosetMap;
 
-  TreeData(void) 
-    : m_outcomeMap(0), m_chanceInfosetMap(0) { }
+  TreeData(void)  { }
   ~TreeData() { }
 };
 
-void ReadPlayers(GameParserState &p_state, 
+void ReadPlayers(GameParserState &p_state,
 		 Game p_game, TreeData &p_treeData)
 {
   if (p_state.GetNextToken() != TOKEN_LBRACE) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+        p_state.CreateLineMsg("Expecting '{' before players"));
   }
 
   while (p_state.GetNextToken() == TOKEN_TEXT) {
     p_game->NewPlayer()->SetLabel(p_state.GetLastText());
-    p_treeData.m_infosetMap.Append(Map<int, GameInfoset>(0));
+    p_treeData.m_infosetMap.Append(std::map<int, GameInfoset>());
   }
 
   if (p_state.GetCurrentToken() != TOKEN_RBRACE) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Expecting '}' after players"));
   }
 }
 
@@ -708,12 +700,13 @@ void ReadPlayers(GameParserState &p_state,
 //                pointing to the 'c', 'p', or 't' token starting the
 //                next node declaration.
 //
-void ParseOutcome(GameParserState &p_state, 
-		  Game p_game, TreeData &p_treeData, 
+void ParseOutcome(GameParserState &p_state,
+		  Game p_game, TreeData &p_treeData,
 		  GameNode p_node)
 {
   if (p_state.GetCurrentToken() != TOKEN_NUMBER) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Expecting index of outcome"));
   }
 
   int outcomeId = atoi(p_state.GetLastText().c_str());
@@ -722,38 +715,41 @@ void ParseOutcome(GameParserState &p_state,
   if (p_state.GetCurrentToken() == TOKEN_TEXT) {
     // This node entry contains information about the outcome
     GameOutcome outcome;
-    if (p_treeData.m_outcomeMap.IsDefined(outcomeId)) {
-      outcome = p_treeData.m_outcomeMap.Lookup(outcomeId);
+    if (p_treeData.m_outcomeMap.count(outcomeId)) {
+      outcome = p_treeData.m_outcomeMap[outcomeId];
     }
     else {
       outcome = p_game->NewOutcome();
-      p_treeData.m_outcomeMap.Define(outcomeId, outcome);
+      p_treeData.m_outcomeMap[outcomeId] = outcome;
     }
 
     outcome->SetLabel(p_state.GetLastText());
     p_node->SetOutcome(outcome);
 
     if (p_state.GetNextToken() != TOKEN_LBRACE) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Expecting '{' before outcome"));
     }
     p_state.GetNextToken();
 
     for (int pl = 1; pl <= p_game->NumPlayers(); pl++) {
       if (p_state.GetCurrentToken() == TOKEN_NUMBER) {
-	outcome->SetPayoff(pl, p_state.GetLastText());
+        outcome->SetPayoff(pl, p_state.GetLastText());
       }
       else {
-	throw InvalidFileException();
+        throw InvalidFileException(
+          p_state.CreateLineMsg("Payoffs should be numbers"));
       }
 
       // Commas are optional between payoffs
       if (p_state.GetNextToken() == TOKEN_COMMA) {
-	p_state.GetNextToken();
+        p_state.GetNextToken();
       }
     }
 
     if (p_state.GetCurrentToken() != TOKEN_RBRACE) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Expecting '}' after outcome"));
     }
     p_state.GetNextToken();
   }
@@ -761,11 +757,12 @@ void ParseOutcome(GameParserState &p_state,
     // The node entry does not contain information about the outcome.
     // This means the outcome better have been defined already;
     // if not, raise an error.
-    if (p_treeData.m_outcomeMap.IsDefined(outcomeId)) {
-      p_node->SetOutcome(p_treeData.m_outcomeMap.Lookup(outcomeId));
+    if (p_treeData.m_outcomeMap.count(outcomeId)) {
+      p_node->SetOutcome(p_treeData.m_outcomeMap[outcomeId]);
     }
     else {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Outcome not defined"));
     }
   }
 }
@@ -779,22 +776,22 @@ void ParseNode(GameParserState &p_state, Game p_game, GameNode p_node,
 // Postcondition: parser state is pointing at the 'c', 'p', or 't'
 //                beginning the next node entry
 //
-void ParseChanceNode(GameParserState &p_state, 
+void ParseChanceNode(GameParserState &p_state,
 		     Game p_game, GameNode p_node, TreeData &p_treeData)
 {
   if (p_state.GetNextToken() != TOKEN_TEXT) {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Expecting label"));
   }
   p_node->SetLabel(p_state.GetLastText());
 
   if (p_state.GetNextToken() != TOKEN_NUMBER) {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Expecting infoset id"));
   }
 
   int infosetId = atoi(p_state.GetLastText().c_str());
   GameInfoset infoset;
-  if (p_treeData.m_chanceInfosetMap.IsDefined(infosetId)) {
-    infoset = p_treeData.m_chanceInfosetMap.Lookup(infosetId);
+  if (p_treeData.m_chanceInfosetMap.count(infosetId)) {
+    infoset = p_treeData.m_chanceInfosetMap[infosetId];
   }
 
   p_state.GetNextToken();
@@ -805,22 +802,23 @@ void ParseChanceNode(GameParserState &p_state,
     std::string label = p_state.GetLastText();
 
     if (p_state.GetNextToken() != TOKEN_LBRACE) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Expecting '{' before information set data"));
     }
     p_state.GetNextToken();
     do {
       if (p_state.GetCurrentToken() != TOKEN_TEXT) {
-	throw InvalidFileException();
+        throw InvalidFileException(p_state.CreateLineMsg("Expecting action"));
       }
       actions.Append(p_state.GetLastText());
 
       p_state.GetNextToken();
-      
+
       if (p_state.GetCurrentToken() == TOKEN_NUMBER) {
-	probs.Append(p_state.GetLastText());
+        probs.Append(p_state.GetLastText());
       }
       else {
-	throw InvalidFileException();
+        throw InvalidFileException(p_state.CreateLineMsg("Expecting probability"));
       }
 
       p_state.GetNextToken();
@@ -829,11 +827,11 @@ void ParseChanceNode(GameParserState &p_state,
 
     if (!infoset) {
       infoset = p_node->AppendMove(p_game->GetChance(), actions.Length());
-      p_treeData.m_chanceInfosetMap.Define(infosetId, infoset);
+      p_treeData.m_chanceInfosetMap[infosetId] = infoset;
       infoset->SetLabel(label);
       for (int act = 1; act <= actions.Length(); act++) {
-	infoset->GetAction(act)->SetLabel(actions[act]);
-	infoset->SetActionProb(act, probs[act]);
+        infoset->GetAction(act)->SetLabel(actions[act]);
+        infoset->SetActionProb(act, probs[act]);
       }
     }
     else {
@@ -846,7 +844,8 @@ void ParseChanceNode(GameParserState &p_state,
   }
   else {
     // Referencing an undefined infoset is an error
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Referencing an undefined infoset"));
   }
 
   ParseOutcome(p_state, p_game, p_treeData, p_node);
@@ -856,31 +855,31 @@ void ParseChanceNode(GameParserState &p_state,
   }
 }
 
-void ParsePersonalNode(GameParserState &p_state, 
+void ParsePersonalNode(GameParserState &p_state,
 		       Game p_game, GameNode p_node, TreeData &p_treeData)
 {
   if (p_state.GetNextToken() != TOKEN_TEXT) {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Expecting label"));
   }
   p_node->SetLabel(p_state.GetLastText());
 
   if (p_state.GetNextToken() != TOKEN_NUMBER) {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Expecting player id"));
   }
   int playerId = atoi(p_state.GetLastText().c_str());
   // This will throw an exception if the player ID is not valid
   GamePlayer player = p_game->GetPlayer(playerId);
-  Map<int, GameInfoset> &infosetMap = p_treeData.m_infosetMap[playerId];
+  std::map<int, GameInfoset> &infosetMap = p_treeData.m_infosetMap[playerId];
 
   if (p_state.GetNextToken() != TOKEN_NUMBER) {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Expecting infoset id"));
   }
   int infosetId = atoi(p_state.GetLastText().c_str());
   GameInfoset infoset;
-  if (infosetMap.IsDefined(infosetId)) {
-    infoset = infosetMap.Lookup(infosetId);
+  if (infosetMap.count(infosetId)) {
+    infoset = infosetMap[infosetId];
   }
-  
+
   p_state.GetNextToken();
 
   if (p_state.GetCurrentToken() == TOKEN_TEXT) {
@@ -889,12 +888,14 @@ void ParsePersonalNode(GameParserState &p_state,
     std::string label = p_state.GetLastText();
 
     if (p_state.GetNextToken() != TOKEN_LBRACE) {
-      throw InvalidFileException();
+      throw InvalidFileException(
+        p_state.CreateLineMsg("Expecting '{' before information set data"));
     }
     p_state.GetNextToken();
     do {
       if (p_state.GetCurrentToken() != TOKEN_TEXT) {
-	throw InvalidFileException();
+        throw InvalidFileException(
+          p_state.CreateLineMsg("Expecting action"));
       }
       actions.Append(p_state.GetLastText());
 
@@ -904,7 +905,7 @@ void ParsePersonalNode(GameParserState &p_state,
 
     if (!infoset) {
       infoset = p_node->AppendMove(player, actions.Length());
-      infosetMap.Define(infosetId, infoset);
+      infosetMap[infosetId] = infoset;
       infoset->SetLabel(label);
       for (int act = 1; act <= actions.Length(); act++) {
 	infoset->GetAction(act)->SetLabel(actions[act]);
@@ -920,7 +921,8 @@ void ParsePersonalNode(GameParserState &p_state,
   }
   else {
     // Referencing an undefined infoset is an error
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Referencing an undefined infoset"));
   }
 
   ParseOutcome(p_state, p_game, p_treeData, p_node);
@@ -930,13 +932,13 @@ void ParsePersonalNode(GameParserState &p_state,
   }
 }
 
-void ParseTerminalNode(GameParserState &p_state, 
+void ParseTerminalNode(GameParserState &p_state,
 		       Game p_game, GameNode p_node, TreeData &p_treeData)
 {
   if (p_state.GetNextToken() != TOKEN_TEXT) {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Expecting label"));
   }
-  
+
   p_node->SetLabel(p_state.GetLastText());
 
   p_state.GetNextToken();
@@ -956,7 +958,7 @@ void ParseNode(GameParserState &p_state, Game p_game, GameNode p_node,
     ParseTerminalNode(p_state, p_game, p_node, p_treeData);
   }
   else {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Invalid type of node"));
   }
 }
 
@@ -964,18 +966,20 @@ void ParseEfg(GameParserState &p_state, Game p_game, TreeData &p_treeData)
 {
   if (p_state.GetNextToken() != TOKEN_NUMBER ||
       p_state.GetLastText() != "2") {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Accepting only EFG version 2"));
   }
 
   if (p_state.GetNextToken() != TOKEN_SYMBOL ||
       (p_state.GetLastText() != "D" && p_state.GetLastText() != "R")) {
-    throw InvalidFileException();
+    throw InvalidFileException(
+      p_state.CreateLineMsg("Accepting only EFG R or D data type"));
   }
   if (p_state.GetNextToken() != TOKEN_TEXT) {
-    throw InvalidFileException();
+    throw InvalidFileException(p_state.CreateLineMsg("Game title missing"));
   }
   p_game->SetTitle(p_state.GetLastText());
-  
+
   ReadPlayers(p_state, p_game, p_treeData);
 
   if (p_state.GetNextToken() == TOKEN_TEXT) {
@@ -990,19 +994,76 @@ void ParseEfg(GameParserState &p_state, Game p_game, TreeData &p_treeData)
 } // end of anonymous namespace
 
 
+#include "tinyxml.h"
+
 namespace Gambit {
 
+class GameXMLSavefile {
+private:
+  TiXmlDocument doc;
+
+public:
+  GameXMLSavefile(const std::string &p_xml);
+  ~GameXMLSavefile()  { }
+
+  Game GetGame(void) const;
+};
+
+GameXMLSavefile::GameXMLSavefile(const std::string &p_xml)
+{
+  doc.Parse(p_xml.c_str());
+  if (doc.Error()) {
+    throw InvalidFileException("Not a valid XML document");
+  }
+}
+
+Game GameXMLSavefile::GetGame(void) const
+{
+  const TiXmlNode *docroot = doc.FirstChild("gambit:document");
+  if (!docroot) {
+    throw InvalidFileException("Not a Gambit game savefile document");
+  }
+
+  const TiXmlNode *game = docroot->FirstChild("game");
+  if (!game) {
+    throw InvalidFileException("No game representation found in document");
+  }
+
+  const TiXmlNode *efgfile = game->FirstChild("efgfile");
+  if (efgfile) {
+    std::istringstream s(efgfile->FirstChild()->Value());
+    return ReadGame(s);
+  }
+  
+  const TiXmlNode *nfgfile = game->FirstChild("nfgfile");
+  if (nfgfile) {
+    std::istringstream s(nfgfile->FirstChild()->Value());
+    return ReadGame(s);
+  }
+
+  throw InvalidFileException("No game representation found in document");
+}
+
 //=========================================================================
 //    ReadGame: Global visible function to read an .efg or .nfg file
 //=========================================================================
 
 Game ReadGame(std::istream &p_file) throw (InvalidFileException)
 {
-  GameParserState parser(p_file);
+  std::stringstream buffer;
+  buffer << p_file.rdbuf();
+  try {
+    GameXMLSavefile doc(buffer.str());
+    return doc.GetGame();
+  }
+  catch (InvalidFileException) {
+    buffer.seekg(0, std::ios::beg);
+  }
 
+  GameParserState parser(buffer);
   try {
     if (parser.GetNextToken() != TOKEN_SYMBOL) {
-      throw InvalidFileException();
+      throw InvalidFileException(parser.CreateLineMsg("Expecting file type"));
     }
 
     if (parser.GetLastText() == "NFG") {
@@ -1012,19 +1073,23 @@ Game ReadGame(std::istream &p_file) throw (InvalidFileException)
     }
     else if (parser.GetLastText() == "EFG") {
       TreeData treeData;
-      Game game(new GameRep);
+      Game game = NewTree();
       ParseEfg(parser, game, treeData);
-      game->Canonicalize();
       return game;
     }
+    else if (parser.GetLastText() == "#AGG") {
+      return GameAggRep::ReadAggFile(p_file);
+    }
+    else if (parser.GetLastText() == "#BAGG") {
+      return GameBagentRep::ReadBaggFile(p_file);
+    }
     else {
-      throw InvalidFileException();
+      throw InvalidFileException("Tokens 'EFG' or 'NFG' or '#AGG' or '#BAGG' expected at start of file");
     }
   }
-  catch (...) {
-    throw InvalidFileException();
+  catch (std::exception &ex) {
+    throw InvalidFileException(ex.what());
   }
 }
 
-
 } // end namespace Gambit
