@@ -1,6 +1,6 @@
 //
 // This file is part of Gambit
-// Copyright (c) 1994-2010, The Gambit Project (http://www.gambit-project.org)
+// Copyright (c) 1994-2014, The Gambit Project (http://www.gambit-project.org)
 //
 // FILE: src/libgambit/game.h
 // Declaration of base class for representing games
@@ -24,6 +24,7 @@
 #ifndef LIBGAMBIT_GAME_H
 #define LIBGAMBIT_GAME_H
 
+#include <memory>
 #include "dvector.h"
 #include "number.h"
 
@@ -48,6 +49,8 @@ public:
   //@{
   /// Constructor; initializes reference count
   GameObject(void) : m_refCount(0), m_valid(true) { }
+  /// Destructor
+  virtual ~GameObject() { }
   //@}
 
   /// @name Validation
@@ -70,14 +73,14 @@ public:
   //@}
 };
 
-/// An exception thrown when attempting to dereference a null pointer
-class NullException : public Exception {
+/// An exception thrown when attempting to dereference an invalidated object 
+class InvalidObjectException : public Exception {
 public:
-  virtual ~NullException() { }
-  std::string GetDescription(void) const
-    { return "Dereferencing null pointer"; }
+  virtual ~InvalidObjectException() throw() { }
+  const char *what(void) const throw()  { return "Dereferencing an invalidated object"; }
 };
 
+
 //
 // This is a handle class that is used by all calling code to refer to
 // member objects of games.  It takes care of all the reference-counting
@@ -104,7 +107,9 @@ public:
     }
 
   T *operator->(void) const 
-    { if (!rep || !rep->IsValid()) throw NullException(); return rep; }
+    { if (!rep) throw NullException();
+      if (!rep->IsValid()) throw InvalidObjectException(); 
+      return rep; }
 
   bool operator==(const GameObjectPtr<T> &r) const
   { return (rep == r.rep); }
@@ -118,73 +123,44 @@ public:
   bool operator!(void) const { return !rep; }
 };
 
-/// A constant forward iterator on an array of GameObjects
-template <class R, class T> class GameObjectIterator {
-private:
-  const Array<R *> &m_array;
-  int m_index;
-
-public:
-  /// @name Lifecycle
-  //@{
-  /// Constructor
-  GameObjectIterator(const Array<R *> &p_array)
-    : m_array(p_array), m_index(m_array.First()) { }
-  //@}
-
-  /// @name Iteration and data access
-  //@{
-  /// Advance to the next element (prefix version)
-  void operator++(void) { m_index++; }
-  /// Advance to the next element (postfix version)
-  void operator++(int) { m_index++; }
-  /// Has iterator gone past the end?
-  bool AtEnd(void) const { return m_index > m_array.Last(); }
-  /// Get the current index into the array
-  int GetIndex(void) const { return m_index; }
-
-  /// Get the current element
-    ///T operator*(void) const { return m_array[m_index]; }
-  /// Get the current element
-  T operator->(void) const { return m_array[m_index]; }
-  /// Get the current element
-  operator T(void) const { return m_array[m_index]; }
-  //@}
-};
-
-
 //
 // Forward declarations of classes defined in this file.
 //
 
+class GameOutcomeRep;
+typedef GameObjectPtr<GameOutcomeRep> GameOutcome;
+
 class GameActionRep;
 typedef GameObjectPtr<GameActionRep> GameAction;
-typedef GameObjectIterator<GameActionRep, GameAction> GameActionIterator;
 
 class GameInfosetRep;
 typedef GameObjectPtr<GameInfosetRep> GameInfoset;
-typedef GameObjectIterator<GameInfosetRep, GameInfoset> GameInfosetIterator;
+class GameTreeInfosetRep;
 
 class GameStrategyRep;
 typedef GameObjectPtr<GameStrategyRep> GameStrategy;
-typedef GameObjectIterator<GameStrategyRep, GameStrategy> GameStrategyIterator;
 
 class GamePlayerRep;
 typedef GameObjectPtr<GamePlayerRep> GamePlayer;
-typedef GameObjectIterator<GamePlayerRep, GamePlayer> GamePlayerIterator;
+typedef Array<GamePlayerRep *> GamePlayers;
 
 class GameNodeRep;
 typedef GameObjectPtr<GameNodeRep> GameNode;
+class GameTreeNodeRep;
 
 class GameRep;
 typedef GameObjectPtr<GameRep> Game;
 
+class PureStrategyProfileRep;
+class PureStrategyProfile;
+
+
 // 
 // Forward declarations of classes defined elsewhere.
 //
 template <class T> class MixedStrategyProfile;
-template <class T> class MixedBehavProfile;
-
+template <class T> class MixedBehaviorProfile;
+class StrategySupportProfile;
 
 //=======================================================================
 //         Exceptions thrown from game representation classes
@@ -193,25 +169,25 @@ template <class T> class MixedBehavProfile;
 /// Exception thrown when an operation that is undefined is attempted
 class UndefinedException : public Exception {
 public:
-  virtual ~UndefinedException() { }
-  std::string GetDescription(void) const
-    { return "Undefined operation on game"; }
+  UndefinedException(void) : Exception("Undefined operation on game") { }
+  UndefinedException(const std::string &s) : Exception(s) { }
+  virtual ~UndefinedException() throw() { }
 };
 
 /// Exception thrown on an operation between incompatible objects
 class MismatchException : public Exception {
 public:
-  virtual ~MismatchException() { }
-  std::string GetDescription(void) const
-    { return "Operation between objects in different games"; }
+  virtual ~MismatchException() throw() { }
+  const char *what(void) const throw()  
+  { return "Operation between objects in different games"; }
 };
 
 /// Exception thrown on a parse error when reading a game savefile
 class InvalidFileException : public Exception {
 public:
-  virtual ~InvalidFileException() { }
-  std::string GetDescription(void) const
-    { return "File not in a recognized format"; }
+  InvalidFileException(void) : Exception("File not in a recognized format") { }
+  InvalidFileException(const std::string &s) : Exception(s) { } 
+  virtual ~InvalidFileException() throw() { }
 };
 
 //=======================================================================
@@ -224,7 +200,9 @@ public:
 /// payoffs are treated as exact (that is, no conversion to floating
 /// point is done).
 class GameOutcomeRep : public GameObject  {
-  friend class GameRep;
+  friend class GameExplicitRep;
+  friend class GameTreeRep;
+  friend class GameTableRep;
   friend class TableFileGameRep;
 
 private:
@@ -232,6 +210,7 @@ private:
   int m_number;
   std::string m_label;
   Array<Number> m_payoffs;
+  GameOutcome m_unrestricted;
 
   /// @name Lifecycle
   //@{
@@ -262,6 +241,10 @@ public:
     m_payoffs[pl] = p_value;
     //m_game->ClearComputedValues();
   }
+
+  /// Map the outcome to the corresponding outcome in the unrestricted game
+  GameOutcome Unrestrict(void) const 
+  { if (m_unrestricted) return m_unrestricted; else throw UndefinedException(); }
   //@}
 };
 
@@ -269,96 +252,75 @@ typedef GameObjectPtr<GameOutcomeRep> GameOutcome;
 
 /// An action at an information set in an extensive game
 class GameActionRep : public GameObject {
-  friend class GameRep;
-  friend class GameInfosetRep;
-  template <class T> friend class MixedBehavProfile;
+protected:
+  GameAction m_unrestricted;
 
-private:
-  int m_number;
-  std::string m_label;
-  GameInfosetRep *m_infoset;
-
-  GameActionRep(int p_number, const std::string &p_label, 
-		  GameInfosetRep *p_infoset)
-    : m_number(p_number), m_label(p_label), m_infoset(p_infoset) { }
-  ~GameActionRep()   { }
+  GameActionRep(void) : m_unrestricted(0) { }
+  virtual ~GameActionRep() { }
 
 public:
-  int GetNumber(void) const { return m_number; }
-  GameInfoset GetInfoset(void) const;
+  virtual int GetNumber(void) const = 0;
+  virtual GameInfoset GetInfoset(void) const = 0;
 
-  const std::string &GetLabel(void) const { return m_label; }
-  void SetLabel(const std::string &p_label) { m_label = p_label; }
+  virtual const std::string &GetLabel(void) const = 0;
+  virtual void SetLabel(const std::string &p_label) = 0;
 
-  bool Precedes(const GameNode &) const;
+  virtual bool Precedes(const GameNode &) const = 0;
+
+  virtual void DeleteAction(void) = 0;
+
+  /// Map the action to the corresponding action in the unrestricted game
+  GameAction Unrestrict(void) const 
+  { if (m_unrestricted) return m_unrestricted; else throw UndefinedException(); }
 
-  void DeleteAction(void);
 };
 
 /// An information set in an extensive game
 class GameInfosetRep : public GameObject {
-  friend class GameRep;
-  friend class GameActionRep;
-  friend class GamePlayerRep;
-  friend class GameNodeRep;
-  template <class T> friend class MixedBehavProfile;
-
 protected:
-  GameRep *m_efg;
-  int m_number;
-  std::string m_label;
-  GamePlayerRep *m_player;
-  Array<GameActionRep *> m_actions;
-  Array<GameNodeRep *> m_members;
-  int flag, whichbranch;
-  Array<Number> m_probs;
-  
-  GameInfosetRep(GameRep *p_efg, int p_number, GamePlayerRep *p_player, 
-		 int p_actions);
-  ~GameInfosetRep();  
-
-  /// Adds the node to the information set
-  void AddMember(GameNodeRep *p_node) { m_members.Append(p_node); }
-  /// Removes the node from the information set, invalidating if emptied
-  void RemoveMember(GameNodeRep *);
-
-  void RemoveAction(int which);
+  GameInfoset m_unrestricted;
+ 
+  GameInfosetRep(void): m_unrestricted(0) { }
+  virtual ~GameInfosetRep() { }
 
 public:
-  Game GetGame(void) const;
-  int GetNumber(void) const { return m_number; }
+  virtual Game GetGame(void) const = 0;
+  virtual int GetNumber(void) const = 0;
   
-  GamePlayer GetPlayer(void) const;
-  void SetPlayer(GamePlayer p);
+  virtual GamePlayer GetPlayer(void) const = 0;
+  virtual void SetPlayer(GamePlayer p) = 0;
 
-  bool IsChanceInfoset(void) const;
+  virtual bool IsChanceInfoset(void) const = 0;
 
-  void SetLabel(const std::string &p_label) { m_label = p_label; }
-  const std::string &GetLabel(void) const { return m_label; }
+  virtual void SetLabel(const std::string &p_label) = 0;
+  virtual const std::string &GetLabel(void) const = 0;
   
-  GameAction InsertAction(GameAction p_where = 0);
+  virtual GameAction InsertAction(GameAction p_where = 0) = 0;
 
   /// @name Actions
   //@{
   /// Returns the number of actions available at the information set
-  int NumActions(void) const { return m_actions.Length(); }
+  virtual int NumActions(void) const = 0;
   /// Returns the p_index'th action at the information set
-  GameAction GetAction(int p_index) const { return m_actions[p_index]; }
+  virtual GameAction GetAction(int p_index) const = 0;
   /// Returns a forward iterator over the available actions
-  GameActionIterator Actions(void) const 
-    { return GameActionIterator(m_actions); }
+  //virtual GameActionIterator Actions(void) const = 0; 
   //@}
 
-  int NumMembers(void) const { return m_members.Length(); }
-  GameNode GetMember(int p_index) const { return m_members[p_index]; }
+  virtual int NumMembers(void) const = 0;
+  virtual GameNode GetMember(int p_index) const = 0;
 
-  bool Precedes(GameNode) const;
+  virtual bool Precedes(GameNode) const = 0;
 
-  void SetActionProb(int i, const std::string &p_value);
-  template <class T> const T &GetActionProb(int pl) const 
-    { return (const T &) m_probs[pl]; }
+  virtual void SetActionProb(int i, const std::string &p_value) = 0;
+  virtual double GetActionProb(int pl, double) const = 0;
+  virtual Rational GetActionProb(int pl, const Rational &) const = 0;
+  virtual std::string GetActionProb(int pl, const std::string &) const = 0;
+  virtual void Reveal(GamePlayer) = 0;
 
-  void Reveal(GamePlayer);
+  /// Map the infoset to the corresponding infoset in the unrestricted game
+  GameInfoset Unrestrict(void) const 
+  { if (m_unrestricted) return m_unrestricted; else throw UndefinedException(); }
 };
 
 /// \brief A strategy in a game.
@@ -371,11 +333,19 @@ public:
 /// find the outcome for that strategy profile, making payoff computation
 /// relatively efficient.
 class GameStrategyRep : public GameObject  {
-  friend class GameRep;
+  friend class GameExplicitRep;
+  friend class GameTreeRep;
+  friend class GameTableRep;
+  friend class GameAggRep;
+  friend class GameBagentRep;
   friend class GamePlayerRep;
-  friend class PureStrategyProfile;
+  friend class PureStrategyProfileRep;
+  friend class TreePureStrategyProfileRep;
+  friend class TablePureStrategyProfileRep;
+  friend class StrategySupportProfile;
   template <class T> friend class MixedStrategyProfile;
-  template <class T> friend class MixedBehavProfile;
+  template <class T> friend class TableMixedStrategyProfileRep;
+  template <class T> friend class MixedBehaviorProfile;
 
 private:
   int m_number, m_id;
@@ -383,12 +353,13 @@ private:
   long m_offset;
   std::string m_label;
   Array<int> m_behav;
+  GameStrategy m_unrestricted;
 
   /// @name Lifecycle
   //@{
   /// Creates a new strategy for the given player.
   GameStrategyRep(GamePlayerRep *p_player)
-    : m_number(0), m_id(0), m_player(p_player), m_offset(0L) { }
+    : m_number(0), m_id(0), m_player(p_player), m_offset(0L), m_unrestricted(0) { }
   //@}
 
 public:
@@ -408,33 +379,46 @@ public:
 
   /// Remove this strategy from the game
   void DeleteStrategy(void);
+
+  /// Map the strategy to the corresponding strategy in the unrestricted game
+  GameStrategy Unrestrict(void) const
+  { if (m_unrestricted) return m_unrestricted; else throw UndefinedException(); }
   //@}
 };
 
+typedef Array<GameStrategyRep *> GameStrategyArray;
+
 /// A player in a game
 class GamePlayerRep : public GameObject {
-  friend class GameRep;
-  friend class GameInfosetRep;
+  friend class GameExplicitRep;
+  friend class GameTreeRep;
+  friend class GameTableRep;
+  friend class GameAggRep;
+  friend class GameBagentRep;
+  friend class GameBaggRep;
+  friend class GameTreeInfosetRep;
   friend class GameStrategyRep;
-  friend class GameNodeRep;
-  template <class T> friend class MixedBehavProfile;
+  friend class GameTreeNodeRep;
+  friend class StrategySupportProfile;
+  template <class T> friend class MixedBehaviorProfile;
   template <class T> friend class MixedStrategyProfile;
 
   /// @name Building reduced form strategies
   //@{
   void MakeStrategy(void);
-  void MakeReducedStrats(GameNodeRep *, GameNodeRep *);
+  void MakeReducedStrats(GameTreeNodeRep *, GameTreeNodeRep *);
   //@}
   
 private:
   GameRep *m_game;
   int m_number;
   std::string m_label;
-  Array<GameInfosetRep *> m_infosets;
-  Array<GameStrategyRep *> m_strategies;
+  Array<GameTreeInfosetRep *> m_infosets;
+  GameStrategyArray m_strategies;
+  GamePlayer m_unrestricted;
 
-  GamePlayerRep(GameRep *p_game, int p_id) : m_game(p_game), m_number(p_id)
-    { }
+  GamePlayerRep(GameRep *p_game, int p_id) 
+    : m_game(p_game), m_number(p_id), m_unrestricted(0) { }
   GamePlayerRep(GameRep *p_game, int p_id, int m_strats);
   ~GamePlayerRep();
 
@@ -452,143 +436,179 @@ public:
   /// Returns the number of information sets at which the player makes a choice
   int NumInfosets(void) const { return m_infosets.Length(); }
   /// Returns the p_index'th information set
-  GameInfoset GetInfoset(int p_index) const { return m_infosets[p_index]; }
-  /// Returns a forward iterator over the information sets
-  GameInfosetIterator Infosets(void) const 
-    { return GameInfosetIterator(m_infosets); }
+  GameInfoset GetInfoset(int p_index) const;
 
   /// @name Strategies
   //@{
   /// Returns the number of strategies available to the player
-  int NumStrategies(void) const { return m_strategies.Length(); }
+  int NumStrategies(void) const; 
   /// Returns the st'th strategy for the player
-  GameStrategy GetStrategy(int st) const { return m_strategies[st]; }
-  /// Returns a forward iterator over the strategies
-  GameStrategyIterator Strategies(void) const 
-    { return GameStrategyIterator(m_strategies); }
+  GameStrategy GetStrategy(int st) const;
+  /// Returns the array of strategies available to the player
+  const GameStrategyArray &Strategies(void) const;
   /// Creates a new strategy for the player
   GameStrategy NewStrategy(void);
   //@}
+
+  /// Map the player to the corresponding player in the unrestricted game
+  GamePlayer Unrestrict(void) const 
+  { if (m_unrestricted) return m_unrestricted; else throw UndefinedException(); }
+
 };
 
 /// A node in an extensive game
 class GameNodeRep : public GameObject {
-  friend class GameRep;
-  friend class GameActionRep;
-  friend class GameInfosetRep;
-  friend class GamePlayerRep;
-  friend class PureBehavProfile;
-  template <class T> friend class MixedBehavProfile;
-  
 protected:
-  int number; 
-  GameRep *m_efg;
-  std::string m_label;
-  GameInfosetRep *infoset;
-  GameNodeRep *m_parent;
-  GameOutcomeRep *outcome;
-  Array<GameNodeRep *> children;
-  GameNodeRep *whichbranch, *ptr;
+  GameNode m_unrestricted;
 
-  GameNodeRep(GameRep *e, GameNodeRep *p);
-  ~GameNodeRep();
-
-  void DeleteOutcome(GameOutcomeRep *outc);
-  void CopySubtree(GameNodeRep *, GameNodeRep *);
+  GameNodeRep(void) : m_unrestricted(0) { }
+  virtual ~GameNodeRep() { }
 
 public:
-  Game GetGame(void) const; 
+  virtual Game GetGame(void) const = 0; 
 
-  const std::string &GetLabel(void) const { return m_label; } 
-  void SetLabel(const std::string &p_label) { m_label = p_label; }
+  virtual const std::string &GetLabel(void) const = 0;
+  virtual void SetLabel(const std::string &p_label) = 0;
 
-  int GetNumber(void) const { return number; }
-  int NumberInInfoset(void) const
-  { return infoset->m_members.Find(const_cast<GameNodeRep *>(this)); }
+  virtual int GetNumber(void) const = 0;
+  virtual int NumberInInfoset(void) const = 0;
 
-  int NumChildren(void) const    { return children.Length(); }
+  virtual int NumChildren(void) const = 0;
 
-  GameInfoset GetInfoset(void) const   { return infoset; }
-  void SetInfoset(GameInfoset);
-  GameInfoset LeaveInfoset(void);
+  virtual GameInfoset GetInfoset(void) const = 0;
+  virtual void SetInfoset(GameInfoset) = 0;
+  virtual GameInfoset LeaveInfoset(void) = 0;
 
-  bool IsTerminal(void) const { return (children.Length() == 0); }
-  GamePlayer GetPlayer(void) const
-    { return (infoset) ? infoset->GetPlayer() : 0; }
-  GameAction GetPriorAction(void) const; // returns null if root node
-  GameNode GetChild(int i) const    { return children[i]; }
-  GameNode GetParent(void) const    { return m_parent; }
-  GameNode GetNextSibling(void) const;
-  GameNode GetPriorSibling(void) const;
+  virtual bool IsTerminal(void) const = 0;
+  virtual GamePlayer GetPlayer(void) const = 0;
+  virtual GameAction GetPriorAction(void) const = 0;
+  virtual GameNode GetChild(int i) const = 0;
+  virtual GameNode GetParent(void) const = 0;
+  virtual GameNode GetNextSibling(void) const = 0;
+  virtual GameNode GetPriorSibling(void) const = 0;
 
-  GameOutcome GetOutcome(void) const { return outcome; }
-  void SetOutcome(const GameOutcome &p_outcome);
+  virtual GameOutcome GetOutcome(void) const = 0;
+  virtual void SetOutcome(const GameOutcome &p_outcome) = 0;
 
-  bool IsSuccessorOf(GameNode from) const;
-  bool IsSubgameRoot(void) const;
+  virtual bool IsSuccessorOf(GameNode from) const = 0;
+  virtual bool IsSubgameRoot(void) const = 0;
 
-  void DeleteParent(void);
-  void DeleteTree(void);
+  virtual void DeleteParent(void) = 0;
+  virtual void DeleteTree(void) = 0;
 
-  void CopyTree(GameNode src);
-  void MoveTree(GameNode src);
+  virtual void CopyTree(GameNode src) = 0;
+  virtual void MoveTree(GameNode src) = 0;
 
-  GameInfoset AppendMove(GamePlayer p_player, int p_actions);
-  GameInfoset AppendMove(GameInfoset p_infoset);
-  GameInfoset InsertMove(GamePlayer p_player, int p_actions);
-  GameInfoset InsertMove(GameInfoset p_infoset);
+  /// Create a separate Game object containing the subgame rooted at the node
+  virtual Game CopySubgame(void) const = 0;
+
+  virtual GameInfoset AppendMove(GamePlayer p_player, int p_actions) = 0;
+  virtual GameInfoset AppendMove(GameInfoset p_infoset) = 0;
+  virtual GameInfoset InsertMove(GamePlayer p_player, int p_actions) = 0;
+  virtual GameInfoset InsertMove(GameInfoset p_infoset) = 0;
+
+  /// Map the node to the corresponding node in the unrestricted game
+  GameNode Unrestrict(void) const 
+  { if (m_unrestricted) return m_unrestricted; else throw UndefinedException(); }
 };
 
 
 /// This class represents a strategy profile on a strategic game.
 /// It specifies exactly one strategy for each player defined on the
 /// game.
-/// Note that, although this is in some sense the analog of a node for
-/// a strategic game, this is not a reference-counted object.
-class PureStrategyProfile  {
-  friend class GameRep;
+class PureStrategyProfileRep {
+  friend class GameTableRep;
+  friend class GameTreeRep;
+  friend class GameAggRep;
+  friend class PureStrategyProfile;
 
-private:
-  long m_index;
+protected:
   Game m_nfg;
   Array<GameStrategy> m_profile;
-  
-public:
-  /// @name Lifecycle
-  //@{
-  /// Construct a new strategy profile on the specified game
-  PureStrategyProfile(const Gambit::Game &);
-  //@}
 
+  /// Construct a new strategy profile
+  PureStrategyProfileRep(void) { }
+
+  /// Create a copy of the strategy profile.
+  /// Caller is responsible for memory management of the created object.
+  virtual PureStrategyProfileRep *Copy(void) const = 0;
+
+public:
+  virtual ~PureStrategyProfileRep() { }
+  
   /// @name Data access and manipulation
   //@{
+  /// Get the index uniquely identifying the strategy profile
+  virtual long GetIndex(void) const { throw UndefinedException(); }
   /// Get the strategy played by player pl  
   const GameStrategy &GetStrategy(int pl) const { return m_profile[pl]; }
   /// Get the strategy played by the player
   const GameStrategy &GetStrategy(const GamePlayer &p_player) const
     { return m_profile[p_player->GetNumber()]; }
   /// Set the strategy for a player
-  void SetStrategy(const GameStrategy &);
+  virtual void SetStrategy(const GameStrategy &) = 0;
 
   /// Get the outcome that results from the profile
-  GameOutcome GetOutcome(void) const;
+  virtual GameOutcome GetOutcome(void) const = 0;
   /// Set the outcome that results from the profile
-  void SetOutcome(GameOutcome p_outcome); 
+  virtual void SetOutcome(GameOutcome p_outcome) = 0; 
 
   /// Get the payoff to player pl that results from the profile
-  template <class T> T GetPayoff(int pl) const;
+  virtual Rational GetPayoff(int pl) const = 0;
   /// Get the payoff to the player resulting from the profile
-  template <class T> T GetPayoff(const GamePlayer &p_player) const
-    { return GetPayoff<T>(p_player->GetNumber()); }
+  Rational GetPayoff(const GamePlayer &p_player) const
+  { return GetPayoff(p_player->GetNumber()); }
   /// Get the value of playing strategy against the profile
-  template <class T> T GetStrategyValue(const GameStrategy &) const;
+  virtual Rational GetStrategyValue(const GameStrategy &) const = 0;
+
+  /// Is the profile a pure strategy Nash equilibrium?
+  bool IsNash(void) const;
+
+  /// Is the profile a strict pure stategy Nash equilibrium?
+  bool IsStrictNash(void) const;
+
+  /// Is the specificed player playing a best response?
+  bool IsBestResponse(const GamePlayer &p_player) const;
+
+  /// Get the list of best response strategies for a player
+  List<GameStrategy> GetBestResponse(const GamePlayer &p_player) const;
+
+  /// Convert to a mixed strategy representation
+  MixedStrategyProfile<Rational> ToMixedStrategyProfile(void) const;
+
+  /// Map strategy profile to the unrestriction of the game
+  PureStrategyProfile Unrestrict(void) const;
+
   //@}
 };
 
+class PureStrategyProfile {
+private:
+  PureStrategyProfileRep *rep;
+
+public:
+  PureStrategyProfile(const PureStrategyProfile &r) : rep(r.rep->Copy())  { }
+  PureStrategyProfile(PureStrategyProfileRep *p_rep) : rep(p_rep) { }
+  ~PureStrategyProfile() { delete rep; }
+
+  PureStrategyProfile &operator=(const PureStrategyProfile &r) 
+    {
+      if (&r != this) {
+	delete rep;
+	rep = r.rep->Copy();
+      }
+      return *this;
+    }
+
+  PureStrategyProfileRep *operator->(void) const { return rep; }
+  operator PureStrategyProfileRep *(void) const { return rep; }
+};
+    
+
 /// This class represents a behavior profile on an extensive game.
 /// It specifies exactly one strategy for each information set in the
 /// game.
-class PureBehavProfile {
+class PureBehaviorProfile {
 private:
   Game m_efg;
   Array<Array<GameAction> > m_profile;
@@ -597,7 +617,7 @@ public:
   /// @name Lifecycle
   //@{
   /// Construct a new behavior profile on the specified game
-  PureBehavProfile(Game);
+  PureBehaviorProfile(Game);
 
   /// @name Data access and manipulation
   //@{
@@ -613,176 +633,178 @@ public:
   template <class T> T GetPayoff(const GamePlayer &p_player) const
     { return GetPayoff<T>(p_player->GetNumber()); }
   /// Get the payoff to player pl conditional on reaching a node
-  template <class T> T GetNodeValue(const GameNode &, int pl) const;
+  template <class T> T GetPayoff(const GameNode &, int pl) const;
   /// Get the payoff to playing the action, conditional on the profile
-  template <class T> T GetActionValue(const GameAction &) const;
+  template <class T> T GetPayoff(const GameAction &) const;
+
+  /// Is the profile a pure strategy agent Nash equilibrium?
+  bool IsAgentNash(void) const;
+
+  /// Convert to a mixed behavior representation
+  MixedBehaviorProfile<Rational> ToMixedBehaviorProfile(void) const;
   //@}
 };
 
 
 /// This is the class for representing an arbitrary finite game.
 class GameRep : public GameObject {
-  friend class GameInfosetRep;
+  friend class GameTreeInfosetRep;
   friend class GamePlayerRep;
-  friend class GameNodeRep;
-  friend class PureStrategyProfile;
-  template <class T> friend class MixedBehavProfile;
+  friend class GameTreeNodeRep;
+  friend class PureStrategyProfileRep;
+  friend class TablePureStrategyProfileRep;
+  template <class T> friend class MixedBehaviorProfile;
   template <class T> friend class MixedStrategyProfile;
+  template <class T> friend class TableMixedStrategyProfileRep;
 
 protected:
-  mutable bool m_computedValues;
   std::string m_title, m_comment;
-  Array<GamePlayerRep *> m_players;
-  GamePlayerRep *m_chance;
 
-  Array<GameOutcomeRep *> m_outcomes;
+  GameRep(void) { }
 
-  GameNodeRep *m_root;
-  Array<GameOutcomeRep *> m_results;
-
-  /// @name Private auxiliary functions
+  /// @name Managing the representation
   //@{
-  void NumberNodes(GameNodeRep *, int &);
-  void IndexStrategies(void);
-  void RebuildTable(void);
+  /// Renumber all game objects in a canonical way
+  virtual void Canonicalize(void) { }  
+  /// Clear out any computed values
+  virtual void ClearComputedValues(void) const { }
+  /// Build any computed values anew
+  virtual void BuildComputedValues(void) { }
+  /// Have computed values been built?
+  virtual bool HasComputedValues(void) const { return false; }
   //@}
 
+
 public:
   /// @name Lifecycle
   //@{
-  /// Construct a new trivial game
-  GameRep(void);
-  /// Construct a new table game with the given dimension
-  GameRep(const Array<int> &p_dim);
   /// Clean up the game
-  virtual ~GameRep();
+  virtual ~GameRep() { }
   /// Create a copy of the game, as a new game
-  Game Copy(void) const;
+  virtual Game Copy(void) const = 0;
   //@}
 
   /// @name General data access
   //@{
   /// Returns true if the game has a game tree representation
-  bool IsTree(void) const { return (m_root != 0); }
+  virtual bool IsTree(void) const = 0;
+
+  /// Returns true if the game has a action-graph game representation
+  virtual bool IsAgg(void) const { return false; }
+
+  /// Returns true if the game is a restriction of a more general game
+  virtual bool IsRestriction(void) const { return false; }
+  /// Returns the unrestricted version of the game
+  virtual Game Unrestrict(void) const { throw UndefinedException(); }
 
   /// Get the text label associated with the game
-  const std::string &GetTitle(void) const { return m_title; }
+  virtual const std::string &GetTitle(void) const { return m_title; }
   /// Set the text label associated with the game
-  void SetTitle(const std::string &p_title) { m_title = p_title; }
+  virtual void SetTitle(const std::string &p_title) { m_title = p_title; }
 
   /// Get the text comment associated with the game
-  const std::string &GetComment(void) const { return m_comment; }
+  virtual const std::string &GetComment(void) const { return m_comment; }
   /// Set the text comment associated with the game
-  void SetComment(const std::string &p_comment) { m_comment = p_comment; }
+  virtual void SetComment(const std::string &p_comment) { m_comment = p_comment; }
 
   /// Returns true if the game is constant-sum
-  bool IsConstSum(void) const; 
+  virtual bool IsConstSum(void) const = 0; 
   /// Returns the smallest payoff in any outcome of the game
-  Rational GetMinPayoff(int pl = 0) const;
+  virtual Rational GetMinPayoff(int pl = 0) const = 0;
   /// Returns the largest payoff in any outcome of the game
-  Rational GetMaxPayoff(int pl = 0) const;
+  virtual Rational GetMaxPayoff(int pl = 0) const = 0;
 
   /// Returns true if the game is perfect recall.  If not, the specified
   /// a pair of violating information sets is returned in the parameters.  
-  bool IsPerfectRecall(GameInfoset &, GameInfoset &) const;
+  virtual bool IsPerfectRecall(GameInfoset &, GameInfoset &) const = 0;
   /// Returns true if the game is perfect recall
-  bool IsPerfectRecall(void) const
-    { GameInfoset s, t; return IsPerfectRecall(s, t); }
-  //@}
-
-  /// @name Managing the representation
-  //@{
-  /// Renumber all game objects in a canonical way
-  void Canonicalize(void);  
-  /// Clear out any computed values
-  void ClearComputedValues(void) const;
-  /// Build any computed values anew
-  void BuildComputedValues(void);
-  /// Have computed values been built?
-  bool HasComputedValues(void) const { return m_computedValues; }
+  virtual bool IsPerfectRecall(void) const
+  { GameInfoset s, t; return IsPerfectRecall(s, t); }
   //@}
 
   /// @name Writing data files
   //@{
-  /// Write the game in .efg format to the specified stream
-  void WriteEfgFile(std::ostream &) const;
-  /// Write the subtree starting at node in .efg format to the specified stream
-  void WriteEfgFile(std::ostream &, const GameNode &p_node) const;
-  /// Write the game in .nfg format to the specified stream
-  void WriteNfgFile(std::ostream &) const;
+  /// Write the game to a savefile in the specified format.
+  virtual void Write(std::ostream &p_stream,
+		     const std::string &p_format="native") const
+  { throw UndefinedException(); }
   //@}
 
   /// @name Dimensions of the game
   //@{
   /// The number of actions in each information set
-  PVector<int> NumActions(void) const;
+  virtual PVector<int> NumActions(void) const = 0;
   /// The number of members in each information set
-  PVector<int> NumMembers(void) const;
+  virtual PVector<int> NumMembers(void) const = 0;
   /// The number of strategies for each player
-  Array<int> NumStrategies(void) const;
+  virtual Array<int> NumStrategies(void) const = 0;
   /// Gets the i'th strategy in the game, numbered globally
-  GameStrategy GetStrategy(int p_index) const;
+  virtual GameStrategy GetStrategy(int p_index) const = 0;
+  /// Returns the number of strategy contingencies in the game
+  virtual int NumStrategyContingencies(void) const = 0;
   /// Returns the total number of actions in the game
-  int BehavProfileLength(void) const;
+  virtual int BehavProfileLength(void) const = 0;
   /// Returns the total number of strategies in the game
-  int MixedProfileLength(void) const;
+  virtual int MixedProfileLength(void) const = 0;
   //@}
 
+  virtual PureStrategyProfile NewPureStrategyProfile(void) const = 0;
+  virtual MixedStrategyProfile<double> NewMixedStrategyProfile(double) const = 0;
+  virtual MixedStrategyProfile<Rational> NewMixedStrategyProfile(const Rational &) const = 0; 
+  virtual MixedStrategyProfile<double> NewMixedStrategyProfile(double, const StrategySupportProfile&) const = 0;
+  virtual MixedStrategyProfile<Rational> NewMixedStrategyProfile(const Rational &, const StrategySupportProfile&) const = 0;
 
   /// @name Players
   //@{
   /// Returns the number of players in the game
-  int NumPlayers(void) const { return m_players.Length(); }
+  virtual int NumPlayers(void) const = 0;
   /// Returns the pl'th player in the game
-  GamePlayer GetPlayer(int pl) const { return m_players[pl]; }
-  /// Returns an iterator over the players
-  GamePlayerIterator Players(void) const
-    { return GamePlayerIterator(m_players); }
+  virtual GamePlayer GetPlayer(int pl) const = 0;
+  /// Returns the set of players in the game 
+  virtual const GamePlayers &Players(void) const = 0;
   /// Returns the chance (nature) player
-  GamePlayer GetChance(void) const { return m_chance; }
+  virtual GamePlayer GetChance(void) const = 0;
   /// Creates a new player in the game, with no moves
-  GamePlayer NewPlayer(void);
+  virtual GamePlayer NewPlayer(void) = 0;
   //@}
 
   /// @name Information sets
   //@{
   /// Returns the iset'th information set in the game (numbered globally)
-  GameInfoset GetInfoset(int iset) const;
+  virtual GameInfoset GetInfoset(int iset) const = 0;
   /// Returns an array with the number of information sets per personal player
-  Array<int> NumInfosets(void) const;
+  virtual Array<int> NumInfosets(void) const = 0;
   /// Returns the act'th action in the game (numbered globally)
-  GameAction GetAction(int act) const;
+  virtual GameAction GetAction(int act) const = 0;
   //@}
 
   /// @name Outcomes
   //@{
   /// Returns the number of outcomes defined in the game
-  int NumOutcomes(void) const { return m_outcomes.Length(); }
+  virtual int NumOutcomes(void) const = 0;
   /// Returns the index'th outcome defined in the game
-  GameOutcome GetOutcome(int index) const { return m_outcomes[index]; }
-
+  virtual GameOutcome GetOutcome(int index) const = 0;
   /// Creates a new outcome in the game
-  GameOutcome NewOutcome(void);
+  virtual GameOutcome NewOutcome(void) = 0;
   /// Deletes the specified outcome from the game
-  void DeleteOutcome(const GameOutcome &);
+  virtual void DeleteOutcome(const GameOutcome &) = 0;
   //@}
 
   /// @name Nodes
   //@{
   /// Returns the root node of the game
-  GameNode GetRoot(void) const 
-  { if (m_root) return m_root; else throw UndefinedException(); }
+  virtual GameNode GetRoot(void) const = 0;
   /// Returns the number of nodes in the game
-  int NumNodes(void) const;
+  virtual int NumNodes(void) const = 0;
   //@}
 };
 
 typedef GameObjectPtr<GameRep> Game;
 
-/// Convenience function to create new game tree
-inline Game NewTree(void) { return new GameRep; }
-/// Convenience function to create new game table
-inline Game NewTable(const Array<int> &p_dim) { return new GameRep(p_dim); }
+/// Factory function to create new game tree
+Game NewTree(void);
+/// Factory function to create new game table
+Game NewTable(const Array<int> &p_dim, bool p_sparseOutcomes = false);
 
 //=======================================================================
 //          Inline members of game representation classes
@@ -793,27 +815,24 @@ inline Game NewTable(const Array<int> &p_dim) { return new GameRep(p_dim); }
 
 inline Game GameOutcomeRep::GetGame(void) const { return m_game; }
 
-inline GameInfoset GameActionRep::GetInfoset(void) const { return m_infoset; }
-
-inline Game GameInfosetRep::GetGame(void) const { return m_efg; }
-inline GamePlayer GameInfosetRep::GetPlayer(void) const { return m_player; }
-inline bool GameInfosetRep::IsChanceInfoset(void) const
-{ return m_player->IsChance(); }
-
 inline GamePlayer GameStrategyRep::GetPlayer(void) const { return m_player; }
 
 inline Game GamePlayerRep::GetGame(void) const { return m_game; }
+inline int GamePlayerRep::NumStrategies(void) const 
+{ m_game->BuildComputedValues(); return m_strategies.Length(); }
+inline GameStrategy GamePlayerRep::GetStrategy(int st) const 
+{ m_game->BuildComputedValues(); return m_strategies[st]; }
+inline const GameStrategyArray &GamePlayerRep::Strategies(void) const
+{ m_game->BuildComputedValues(); return m_strategies; }
 
-inline Game GameNodeRep::GetGame(void) const { return m_efg; }
+template<> inline double PureBehaviorProfile::GetPayoff(int pl) const
+{ return GetPayoff<double>(m_efg->GetRoot(), pl); }
 
-template<> inline double PureBehavProfile::GetPayoff(int pl) const
-{ return GetNodeValue<double>(m_efg->GetRoot(), pl); }
+template<> inline Rational PureBehaviorProfile::GetPayoff(int pl) const
+{ return GetPayoff<Rational>(m_efg->GetRoot(), pl); }
 
-template<> inline Rational PureBehavProfile::GetPayoff(int pl) const
-{ return GetNodeValue<Rational>(m_efg->GetRoot(), pl); }
-
-template<> inline std::string PureBehavProfile::GetPayoff(int pl) const
-{ return ToText(GetNodeValue<Rational>(m_efg->GetRoot(), pl)); }
+template<> inline std::string PureBehaviorProfile::GetPayoff(int pl) const
+{ return lexical_cast<std::string>(GetPayoff<Rational>(m_efg->GetRoot(), pl)); }
 
 //=======================================================================
 
