@@ -1,6 +1,6 @@
 //
 // This file is part of Gambit
-// Copyright (c) 1994-2010, The Gambit Project (http://www.gambit-project.org)
+// Copyright (c) 1994-2014, The Gambit Project (http://www.gambit-project.org)
 //
 // FILE: src/tools/enumpoly/pelclass.cc
 // Implementation of interface to Pelican
@@ -20,6 +20,7 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //
 
+#include <cstdlib>
 #include "pelclass.h"
 
 /*
@@ -28,6 +29,7 @@
 #include "complex.h"
 */
 
+inline double abs(double x) { return fabs(x); }
 
 /*************************************************************/
 /************** Implementation of class Pelview **************/
@@ -48,16 +50,16 @@ void PelView::InitializePelicanMemory() const
 
 Pring PelView::MakePring(const int num) const
 {
-  char* q[] = {"","n1","n2" , "n3", "n4", "n5", "n6", "n7", "n8", 
-	       "n9", "n10", "n11", "n12", "n13", "n14", "n15", "n16", 
-	       "n17", "n18","n19", "n20", "n21", "n22", "n23", "n24", 
-	       "n25", "n26", "n27", "n28","n29", "n30"};
-
+  const char* q[] = {"","n1","n2" , "n3", "n4", "n5", "n6", "n7", "n8", 
+		     "n9", "n10", "n11", "n12", "n13", "n14", "n15", "n16", 
+		     "n17", "n18","n19", "n20", "n21", "n22", "n23", "n24", 
+		     "n25", "n26", "n27", "n28","n29", "n30"};
+  
   Pring R = makePR(num);    // makePR is a memory malloc routine
 
   for(int j=1; j<=R->n; j++) 
-    R->vars[j-1] = q[j];
-  R->def = "t";
+    R->vars[j-1] = const_cast<char *>(q[j]);
+  R->def = const_cast<char *>("t");
   R->n = num;
   return R;
 }
@@ -85,10 +87,10 @@ void PelView::Initialize_Idf_T_Gen_node(const Gen_node &node,
 
 Gen_node PelView::CreateRing(const int numvar) const
 {
-  char* q[] = {" ", "","n1","n2" , "n3", "n4", "n5", "n6", "n7", "n8", 
-	       "n9", "n10", "n11", "n12", "n13", "n14", "n15", "n16", 
-	       "n17", "n18","n19", "n20", "n21", "n22", "n23", "n24", 
-	       "n25", "n26", "n27", "n28","n29", "n30"};
+  const char* q[] = {" ", "","n1","n2" , "n3", "n4", "n5", "n6", "n7", "n8", 
+		     "n9", "n10", "n11", "n12", "n13", "n14", "n15", "n16", 
+		     "n17", "n18","n19", "n20", "n21", "n22", "n23", "n24", 
+		     "n25", "n26", "n27", "n28","n29", "n30"};
 
   Gen_node a1 = gen_node();
   Initialize_Idf_T_Gen_node(a1,"n1");
@@ -305,8 +307,8 @@ Gen_node PelView::Make_scl_Gen_node() const
 
   g->type= Idf_T;
   g->next= NULL;
-  g->Genval.gval= "scl";
-  g->Genval.idval= "scl";
+  g->Genval.gval= const_cast<char *>("scl");
+  g->Genval.idval= const_cast<char *>("scl");
 
   return g;
 }
@@ -540,7 +542,7 @@ PelView::RealRoots(const Gambit::List<Gambit::Vector<gComplex> > &clist) const
 
     bool is_real = true;
     for (int j = 1; j <= Dmnsn(); j++)
-      if (Gambit::abs(clist[i][j].ImaginaryPart()) > 0.0001) 
+      if (abs(clist[i][j].ImaginaryPart()) > 0.0001) 
 	is_real = false;
 
     if (is_real) {
@@ -560,7 +562,7 @@ bool PelView::CheckSolutions(const Gen_node g) const
   Gen_node goo;
   goo = g->Genval.lval;
   while (goo!=0) { 
-    if (Gambit::abs(goo->Genval.dval) > 0.01) 
+    if (abs(goo->Genval.dval) > 0.01) 
       return 0;
 
     goo = goo->next;
